// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
/*
   C interface definition for DAL

   (c) 1997 European Space Agency - written by Mark Thomas, CARA

*/


#include <cstdio>
#include "Tracer.h"
#include "Dal.h"
#include "dal_utilities.h"


extern "C" {



Array *
addArray(
		DataSet *			dataSet,
		const char *			name,
		Dal::DataType			dataType,
		int				numberofDimensions,
		unsigned long *			dimensions,
		const char *			units,
		const char *			comment,
		int				position )
{
	PROC
	RETURN dataSet -> addArray(
				name,
				(Dal::DataType)dataType,
				vector<unsigned long>(
					dimensions,
					dimensions + numberofDimensions ),
				units,
				comment,
				position );


}


Column *
addColumn(
		Table *				table,
		const char *			name,
		Dal::DataType			dataType,
		const char *			units,
		const char *			comment,
		int				numberofDimensions,
		unsigned long *			dimensions,
		int				position )
{

	PROC
	RETURN table -> addColumn(
				name,
				(Dal::DataType)(dataType),
				units,
				comment,
				vector<unsigned long>(
					dimensions,
					dimensions + numberofDimensions ),
				position );

}


void
addCommentToArray(
		Array *				array,
		const char *			comment )
{
	PROC
	array -> addComment( comment );
	ENDPROC
}


void
addCommentToAttributable(
		Attributable *			attributable,
		const char *			comment )
{
	PROC
	attributable -> addComment( comment );
	ENDPROC
}


void
addCommentToBlock(
		Block *				block,
		const char *			comment )
{
	PROC
	block -> addComment( comment );
	ENDPROC
}


void
addCommentToDataSet(
		DataSet *			dataSet,
		const char *			comment )
{
	PROC
	dataSet -> addComment( comment );
	ENDPROC
}


void
addCommentToTable(
		Table *				table,
		const char *			comment )
{
	PROC
	table -> addComment( comment );
	ENDPROC
}


void
addHistoryToArray(
		Array *				array,
		const char *			text )
{
	PROC
	array -> addHistory( text );
	ENDPROC
}


void
addHistoryToAttributable(
		Attributable *			attributable,
		const char *			text )
{
	PROC
	attributable -> addHistory( text );
	ENDPROC
}


void
addHistoryToBlock(
		Block *				block,
		const char *			text )
{
	PROC
	block -> addHistory( text );
	ENDPROC
}


void
addHistoryToDataSet(
		DataSet *			dataSet,
		const char *			text )
{
	PROC
	dataSet -> addHistory( text );
	ENDPROC
}


void
addHistoryToTable(
		Table *				table,
		const char *			text )
{
	PROC
	table -> addHistory( text );
	ENDPROC
}


Table *
addTable(
		DataSet *		dataSet,
		const char *		name,
		int 			numberOfRows,
		const char *		comment,
		int 			position )
{
	PROC
	RETURN dataSet -> addTable( name, numberOfRows, comment, position );

}



const char *
arrayAttributeComment(
		const Array *			array,
		const char *			name )
{
	PROC
	RETURN array -> attribute( name ) -> label().c_str();

}


Attribute::DataType
arrayAttributeDataTypeWithName(
		const Array *			array,
		const char *			name )
{
	PROC
	RETURN array -> attribute( name ) -> dataType();
}


Attribute::DataType
arrayAttributeDataTypeWithNumber(
		const Array *			array,
		unsigned int			number )
{
	PROC
	RETURN array -> attribute( number ) -> dataType();
}


const char *
arrayAttributeUnits(
		const Array *			array,
		const char *			name )
{
	PROC
	RETURN array -> attribute( name ) -> units().c_str();

}


const Attribute *
arrayAttributeWithName(
		const Array *			array,
		const char *			name )
{
	PROC
	RETURN array -> attribute( name );
}


const Attribute *
arrayAttributeWithNumber(
		const Array *			array,
		unsigned int			number )
{
	PROC
	RETURN array -> attribute( number );
}


Dal::DataType
arrayDataType( const Array *		array )
{
	PROC
	RETURN array -> dataType();
}


bool8
arrayHasAttribute(
	const Array *			array,
	const char *			name )
{
	PROC
	RETURN array -> hasAttribute( name );
}


const char *
arrayLabel(
	const Array *			array )
{
	PROC
	RETURN array -> label().c_str();
}


Attributable *
arrayToAttributable(
		Array *				array )
{
	PROC
	RETURN array;
}


Block *
arrayToBlock(
	Array *			array )
{
	PROC
	RETURN array;
}


Nullable *
arrayToNullable( Array * array )
{
	PROC
	RETURN array;
}


const char *
arrayUnits(
	const Array *		array )
{
	PROC
	RETURN array -> units().c_str();
}


Array *
arrayWithName(
	const DataSet *		dataSet,
	const char *		name )
{
	PROC
	RETURN const_cast<Array *>( dataSet -> array( name ));
}


Array *
arrayWithNumber(
	const DataSet *		dataSet,
	unsigned int		position )
{
	PROC
	RETURN const_cast<Array *>( dataSet -> array( position ));
}


Attribute::DataType
attributableAttributeDataTypeWithName(
		const Attributable *		attributable,
		const char *			name )
{
	PROC
	RETURN attributable -> attribute( name ) -> dataType();
}


Attribute::DataType
attributableAttributeDataTypeWithNumber(
		const Attributable *		attributable,
		unsigned int			number )
{
	PROC
	RETURN attributable -> attribute( number ) -> dataType();
}


const Attribute *
attributableAttributeWithName(
		const Attributable *		attributable,
		const char *			name )
{
	PROC
	RETURN attributable -> attribute( name );
}


const Attribute *
attributableAttributeWithNumber(
		const Attributable *		attributable,
		unsigned int			number )
{
	PROC
	RETURN attributable -> attribute( number );
}



Attribute::DataType
attributeDataType(
		Attribute *			attribute )
{
	PROC
	RETURN attribute -> dataType();
}
		

const char *
attributeLabel(
		const Attribute *		attribute )
{
	PROC
	RETURN attribute -> label().c_str();

}


const char *
attributeUnits(
		const Attribute *		attribute )
{
	PROC
	RETURN attribute -> units().c_str();

}


const char *
blockAttributeComment(
		const Block *			block,
		const char *			name )
{
	PROC
	RETURN block -> attribute( name ) -> label().c_str();
}


Attribute::DataType
blockAttributeDataTypeWithName(
		const Block *			block,
		const char *			name )
{
	PROC
	RETURN block -> attribute( name ) -> dataType();
}


Attribute::DataType
blockAttributeDataTypeWithNumber(
		const Block *			block,
		unsigned int			number )
{
	PROC
	RETURN block -> attribute( number ) -> dataType();
}


const char *
blockAttributeUnits(
	const Block *			block,
	const char *			name )
{
	PROC
	RETURN block -> attribute( name ) -> units().c_str();
}


Attribute *
blockAttributeWithName(
		Block *				block,
		const char *			name )
{
	PROC
	RETURN block -> attribute( name );
}


Attribute *
blockAttributeWithNumber(
		Block *				block,
		unsigned int			number )
{
	PROC
	RETURN block -> attribute( number );
}


bool8
blockHasAttribute(
	const Block *			block,
	const char *			name )
{
	PROC
	RETURN block -> hasAttribute( name );
}


const char *
blockLabel(
	const Block *			block )
{
	PROC
	RETURN block -> label().c_str();
}


unsigned int
blockNumber(
	const DataSet *			dataSet,
	const char *			name )
{
	PROC
	RETURN dataSet -> blockNumber( name );
}


Attributable *
blockToAttributable(
		Block *				block )
{
	PROC
	RETURN block;
}


Block::Type
blockType( const Block *		block )
{
	PROC
	RETURN block -> type();
}


Block *
blockWithName(
	const DataSet *		dataSet,
	const char *		name )
{
	PROC
	RETURN const_cast<Block *>( dataSet -> block( name ));
}


Block *
blockWithNumber(
	const DataSet *		dataSet,
	unsigned int		position )
{
	PROC
	RETURN const_cast<Block *>( dataSet -> block( position ));
}


bool8
boolArrayAttribute(
	const Array *			array,
	const char *			name )
{
	PROC
	RETURN array -> attribute( name ) -> asBool();

}


bool8 *
boolArrayData(
	const Array *			array )
{
	PROC
	RETURN array -> data() -> boolData();
}


bool8
boolAttribute(
		const Attribute *	attribute )
{
	PROC
	RETURN attribute -> asBool();
}


bool8
boolBlockAttribute(
	const Block *			block,
	const char *			name )
{
	PROC
	RETURN block -> attribute( name ) -> asBool();

}


bool8
boolColumnAttribute(
	const Column *			column,
	const char *			name )
{
	PROC
	RETURN column -> attribute( name ) -> asBool();

}


bool8 *
boolColumnData(
	const Column *			column )
{
	PROC
	RETURN column -> data() -> boolData();
}


bool8
boolDataSetAttribute(
	const DataSet *			dataSet,
	const char *			name )
{
	PROC
	RETURN dataSet -> attribute( name ) -> asBool();

}


bool8
boolTableAttribute(
	const Table *			table,
	const char *			name )
{
	PROC
	RETURN table -> attribute( name ) -> asBool();

}


int
cellSize(
		Column *			column,
		int				rowNumber )
{
	PROC

	CellData * cellData = column -> cellData( rowNumber );
	int size = cellData -> size();
	delete cellData;

	RETURN size;
}


Column::CellType
cellType(
		Column *			column )
{
        PROC
        RETURN column -> cellType();
}


bool8
clobber()
{
	PROC
	RETURN dataSetServer -> clobber();
}


const char *
columnAttributeComment(
		const Column *			column,
		const char *			name )
{
	PROC
	RETURN column -> attribute( name ) -> label().c_str();
}


Attribute::DataType
columnAttributeDataTypeWithName(
		const Column *			column,
		const char *			name )
{
	PROC
	RETURN column -> attribute( name ) -> dataType();
}


Attribute::DataType
columnAttributeDataTypeWithNumber(
		const Column *			column,
		unsigned int			number )
{
	PROC
	RETURN column -> attribute( number ) -> dataType();
}


const char *
columnAttributeUnits(
	const Column *			column,
	const char *			name )
{
	PROC
	RETURN column -> attribute( name ) -> units().c_str();
}


const Attribute *
columnAttributeWithName(
		const Column *			column,
		const char *			name )
{
	PROC
	RETURN column -> attribute( name );
}


const Attribute *
columnAttributeWithNumber(
		const Column *			column,
		unsigned int			number )
{
	PROC
	RETURN column -> attribute( number );
}


unsigned long
columnDataIndex(
	const Column *			column,
	unsigned long			rowNumber,
	unsigned int			numberOfDimensions,
	const unsigned long *		dimensions )
{
	PROC
	const ColumnData *	columnData = column -> data();
	unsigned 		index = columnData -> position(
							rowNumber,
							vector<unsigned long>(
								dimensions,
								dimensions + numberOfDimensions ));
	delete columnData;

	RETURN index;
}


Dal::DataType
columnDataType(
	const Column *			column )
{
	PROC
	RETURN column -> dataType();
}


bool8
columnHasAttribute(
	const Column *			column,
	const char *			name )
{
	PROC
	RETURN column -> hasAttribute( name );
}


unsigned int
columnNumber(
	const Table *			table,
	const char *			name )
{
	PROC
	RETURN table -> columnNumber( name );
}


const char *
columnLabel(
	const Column *			column )
{
	PROC
	RETURN column -> label().c_str();
}


Attributable *
columnToAttributable(
		Column *			column )
{
	PROC
	RETURN column;
}


Nullable *
columnToNullable( Column * column )
{
	PROC
	RETURN column;
}


const char *
columnUnits(
	const Column *			column )
{
	PROC
	RETURN column -> units().c_str();
}


Column *
columnWithName(
		const Table *			table,
		const char *			name )
{
	PROC
	RETURN const_cast<Column *>( table -> column( name ));
}


Column *
columnWithNumber(
		const Table *			table,
		int				position )
{
	PROC
	RETURN const_cast<Column *>( table -> column( position ));
}


const char *
commentOfArray(
	Array *			array,
	unsigned int		number)
{
	PROC
	RETURN array -> comment( number ).c_str();
}


const char *
commentOfAttributable(
		Attributable *			attributable,
		unsigned int 			number )
{
	PROC
	RETURN attributable -> comment( number ).c_str();
}


const char *
commentOfBlock(
		Block *				block,
		unsigned int 			number )
{
	PROC
	RETURN block -> comment( number ).c_str();
}


const char *
commentOfColumn(
		Column *			column,
		unsigned int 			number )
{
	PROC
	RETURN column -> comment( number ).c_str();
}


const char *
commentOfDataSet(
	const DataSet *		dataSet,
	unsigned int		number)
{
	PROC
	RETURN dataSet -> comment( number ).c_str();
}


const char *
commentOfTable(
	Table *			table,
	unsigned int		number)
{
	PROC
	RETURN ((Attributable *)table) -> comment( number ).c_str();
}


void
copyAttributesOfArray(
	Array *				to,
	const Array *			from,
	CopyMode			copyMode )
{
	PROC
	copyAttributes( *to, *from, copyMode );
	ENDPROC
}


void
copyAttributesOfBlock(
	Block *				to,
	const Block *			from,
	CopyMode			copyMode )
{
	PROC
	copyAttributes( *to, *from, copyMode );
	ENDPROC
}


void
copyAttributesOfColumn(
	Column *			to,
	const Column *			from,
	CopyMode			copyMode )
{
	PROC
	copyAttributes( *to, *from, copyMode );
	ENDPROC
}


void
copyAttributesOfDataSet(
	DataSet *			to,
	const DataSet *			from,
	CopyMode			copyMode )
{
	PROC
	copyAttributes( *to, *from, copyMode );
	ENDPROC
}


void
copyAttributesOfTable(
	Table *				to,
	const Table *			from,
	CopyMode			copyMode )
{
	PROC
	copyAttributes( *to, *from, copyMode );
	ENDPROC
}


void
copyAttributeToArray(
	Array *				array,
	const Attribute *		attribute )
{
	PROC
	array -> addAttribute( attribute );
	ENDPROC
}


void
copyAttributeToBlock(
	Block *				block,
	const Attribute *		attribute )
{
	PROC
	block -> addAttribute( attribute );
	ENDPROC
}


void
copyAttributeToColumn(
	Column *			column,
	const Attribute *		attribute )
{
	PROC
	column -> addAttribute( attribute );
	ENDPROC
}


void
copyAttributeToDataSet(
	DataSet *			dataSet,
	const Attribute *		attribute )
{
	PROC
	dataSet -> addAttribute( attribute );
	ENDPROC
}


void
copyAttributeToTable(
	Table *				table,
	const Attribute *		attribute )
{
	PROC
	table -> addAttribute( attribute );
	ENDPROC
}


void
copyRows(
		Table *				table,
		unsigned int			from,
		unsigned int			to,
		unsigned long			count )
{
	PROC
	table -> copyRows( from, to, count );
	ENDPROC
}


Nullable *
dataComponentToNullable( DataComponent * dataComponent )
{
	PROC
	RETURN dataComponent;
}


void
dataServerClient(
		 const char *                   clientName )
{
	PROC
	dataSetServer -> client( clientName );
	ENDPROC
}


DataSet *
dataSet(
		const char *			dataSetName,
		Dal::AccessMode			openMode,
		DataSetServer::MemoryModel	memoryModel )
{
	PROC
	RETURN dataSetServer -> open( dataSetName, openMode, memoryModel );
}


const char *
dataSetAttributeComment(
		const DataSet *			dataSet,
		const char *			name )
{
	PROC
	RETURN dataSet -> attribute( name ) -> label().c_str();
}


Attribute::DataType
dataSetAttributeDataTypeWithName(
		const DataSet *			dataSet,
		const char *			name )
{
	PROC
	RETURN dataSet -> attribute( name ) -> dataType();
}


Attribute::DataType
dataSetAttributeDataTypeWithNumber(
		const DataSet *			dataSet,
		unsigned int			number )
{
	PROC
	RETURN dataSet -> attribute( number ) -> dataType();
}


const char *
dataSetAttributeUnits(
	const DataSet *			dataSet,
	const char *			name )
{
	PROC
	RETURN dataSet -> attribute( name ) -> units().c_str();
}


const Attribute *
dataSetAttributeWithName(
		const DataSet *			dataSet,
		const char *			name )
{
	PROC
	RETURN dataSet -> attribute( name );
}


Attribute *
dataSetAttributeWithNumber(
		DataSet *			dataSet,
		unsigned int			number )
{
	PROC
	RETURN dataSet -> attribute( number );
}


bool8
dataSetHasAttribute(
	const DataSet *			dataSet,
	const char *			name )
{
	PROC
	RETURN dataSet -> hasAttribute( name );
}


bool8
dataSetHasBlock(
		const DataSet * dataSet,
		const char * name)
{
	PROC
	RETURN dataSet -> hasBlock( name );
}


Attributable *
dataSetToAttributable(
		DataSet *			dataSet )
{
	PROC
	RETURN dataSet;
}


void
deleteArrayAttributeWithName(
                Array *                         array,
                const char *                    name )
{
	PROC
	array -> deleteAttribute( name );
	ENDPROC
}


void
deleteArrayAttributeWithNumber(
                Array *                         array,
                unsigned int                    number )
{
	PROC
	array -> deleteAttribute( number );
	ENDPROC
}


void
deleteAttribute(
                Attribute *                     attribute )
{
	PROC
	attribute -> parent() -> deleteAttribute( attribute -> name() );
	ENDPROC
}


void
deleteBlockAttributeWithName(
                Block *                         block,
                const char *                    name )
{
	PROC
	block -> deleteAttribute( name );
	ENDPROC
}


void
deleteBlockAttributeWithNumber(
                Block *                         block,
                unsigned int                    number )
{
	PROC
	block -> deleteAttribute( number );
	ENDPROC
}


void
deleteBlockWithName(
	DataSet *			dataSet,
	const char *			name )
{
	PROC
	dataSet -> deleteBlock( name );
	ENDPROC
}


void
deleteBlockWithNumber(
	DataSet *			dataSet,
	unsigned int			position )
{
	PROC
	dataSet -> deleteBlock( position );
	ENDPROC
}


void
deleteColumnAttributeWithName(
                Column *                        column,
                const char *                    name )
{
	PROC
	column -> deleteAttribute( name );
	ENDPROC
}


void
deleteColumnAttributeWithNumber(
                Column *                        column,
                unsigned int                    number )
{
	PROC
	column -> deleteAttribute( number );
	ENDPROC
}


void
deleteColumnWithName(
	Table *				table,
	const char *			name )
{
	PROC
	table -> deleteColumn( name );
	ENDPROC
}


void
deleteColumnWithNumber(
	Table *				table,
	unsigned int			position )
{
	PROC
	table -> deleteColumn( position );
	ENDPROC
}


void
deleteDataSetAttributeWithName(
                DataSet *                       dataSet,
                const char *                    name )
{
	PROC
	dataSet -> deleteAttribute( name );
	ENDPROC
}


void
deleteDataSetAttributeWithNumber(
                DataSet *                       dataSet,
                unsigned int                    number )
{
	PROC
	dataSet -> deleteAttribute( number );
	ENDPROC
}


void
deleteRows(
		Table *				table,
		unsigned int			from,
		unsigned int			count )
{
	PROC
	table -> deleteRows( from, count );
	ENDPROC
}


void
deleteTableAttributeWithName(
                Table *                         table,
                const char *                    name )
{
	PROC
	table -> deleteAttribute( name );
	ENDPROC
}


void
deleteTableAttributeWithNumber(
                Table *                         table,
                unsigned int                    number )
{
	PROC
	table -> deleteAttribute( number );
	ENDPROC
}


unsigned long *
dimensionsOfArray(
	const Array *			array )
{
	PROC
	// *****************************************************
	// The memory will have to be deallocated by the caller.
	// *****************************************************
	  /* AI gcc4 porting
	   * array bound forbidden after parenthesized type-id
	   *  try removing the parentheses around the type-id
	   unsigned long * size = new (unsigned long)[ array -> dimensions() ]; =>
	   unsigned long * size = new unsigned long[ array -> dimensions() ];
	  */
	unsigned long * size = new unsigned long[ array -> dimensions() ];
	for( unsigned int i = 0; i < array -> dimensions(); ++i )
		size[ i ] = array -> size()[ i ];

	RETURN size;
}


unsigned long *
dimensionsOfColumn(
	const Column *			column )
{
	PROC
	// *****************************************************
	// The memory will have to be deallocated by the caller.
	// *****************************************************
	  /* AI gcc4 porting
	   * array bound forbidden after parenthesized type-id
	   *  try removing the parentheses around the type-id
	   * unsigned long * size = new (unsigned long)[ column -> dimensions() ]; =>
	   * unsigned long * size = new unsigned long[ column -> dimensions() ];
	  */
	  unsigned long * size = new unsigned long[ column -> dimensions() ];
	for( unsigned int i = 0; i < column -> dimensions(); ++i )
		size[ i ] = column -> size()[ i ];

	RETURN size;
}

void
discardDataSet(
	const char *		dataSetName )
{
	PROC
	dataSetServer -> discard( dataSetName );
	ENDPROC
}


typedef void(*TableIteratorFunction)( Table * ); 


void
forEachSubTable(
	Table *			table,
	TableIteratorFunction 	callThisFunction )
{
	PROC
	table -> forEachSubTable( callThisFunction );
	ENDPROC
}


bool8
hasNullsArray(
		const Array *			array )
{
	PROC
	ArrayData * ad = array -> data();
	const int hasNulls = ad -> hasNulls();
	delete ad;
	RETURN hasNulls;
}


bool8
hasNullsColumn(
		const Column *			column )
{
	PROC
	ColumnData * cd = column -> data();
	const int hasNulls = cd -> hasNulls();
	delete cd;
	RETURN hasNulls;
}


bool8
hasScalingOfArray(
		const Array *		array )
{
	PROC
	RETURN array -> scaled();
}


bool8
hasScalingOfColumn(
		const Column *		column )
{
	PROC
	RETURN column -> scaled();
}


const char *
historyOfArray(
	Array * 		array,
	unsigned int 		number)
{
	PROC
	RETURN array -> history( number ).c_str();
}


const char *
historyOfBlock(
	Block * 		block,
	unsigned int 		number)
{
	PROC
	RETURN block -> history( number ).c_str();
}


const char *
historyOfDataSet(
	DataSet * 		dataSet,
	unsigned int 		number)
{
	PROC
	RETURN dataSet -> history( number ).c_str();
}


const char *
historyOfTable(
	Table * 		table,
	unsigned int 		number)
{
	PROC
	RETURN table -> history( number ).c_str();
}


int8
int8ArrayAttribute(
		const Array *		array,
		const char *		name )
{
	PROC
	RETURN array -> attribute( name ) -> asInt();
}


int8 *
int8ArrayData(
		const Array *		array )
{
	PROC
	RETURN array -> data() -> int8Data();
}


int8
int8Attribute(
		const Attribute *	attribute )
{
	PROC
	RETURN attribute -> asInt();
}


int8
int8BlockAttribute(
		const Block *		block,
		const char *		name )
{
	PROC
	RETURN block -> attribute( name ) -> asInt();
}


int8
int8ColumnAttribute(
		const Column *		column,
		const char *		name )
{
	PROC
	RETURN column -> attribute( name ) -> asInt();
}


int8 *
int8ColumnData(
		const Column *		column )
{
	PROC
	RETURN column -> data() -> int8Data();
}


int8
int8DataSetAttribute(
		const DataSet *		dataSet,
		const char *		name )
{
	PROC
	RETURN dataSet -> attribute( name ) -> asInt();

}


int8
int8TableAttribute(
		const Table *		table,
		const char *		name )
{
	PROC
	RETURN table -> attribute( name ) -> asInt();
}


int16
int16ArrayAttribute(
		const Array *		array,
		const char *		name )
{
	PROC
	RETURN array -> attribute( name ) -> asInt();
}


int16 *
int16ArrayData(
		const Array *		array )
{
	PROC
	RETURN array -> data() -> int16Data();
}


int16
int16Attribute(
		const Attribute *	attribute )
{
	PROC
	RETURN attribute -> asInt();
}


int16
int16BlockAttribute(
		const Block *		block,
		const char *		name )
{
	PROC
	RETURN block -> attribute( name ) -> asInt();
}


int16
int16ColumnAttribute(
		const Column *		column,
		const char *		name )
{
	PROC
	RETURN column -> attribute( name ) -> asInt();
}


int16 *
int16ColumnData(
		const Column *		column )
{
	PROC
	RETURN column -> data() -> int16Data();
}

int16
int16DataSetAttribute(
		const DataSet *		dataSet,
		const char *		name )
{
	PROC
	RETURN dataSet -> attribute( name ) -> asInt();

}


int16
int16TableAttribute(
		const Table *		table,
		const char *		name )
{
	PROC
	RETURN table -> attribute( name ) -> asInt();
}


int32
int32ArrayAttribute(
		const Array *		array,
		const char *		name )
{
	PROC
	RETURN array -> attribute( name ) -> asInt();
}


int32 *
int32ArrayData(
		const Array *		array )
{
	PROC
	RETURN array -> data() -> int32Data();
}


int32
int32AttributableAttribute(
		const Attributable *	attributable,
		const char *		name )
{
	PROC
	RETURN attributable -> attribute( name ) -> asInt();
}


int32
int32Attribute(
		const Attribute *	attribute )
{
	PROC
	RETURN attribute -> asInt();
}


int32
int32BlockAttribute(
		const Block *		block,
		const char *		name )
{
	PROC
	RETURN block -> attribute( name ) -> asInt();
}


int32
int32ColumnAttribute(
		const Column *		column,
		const char *		name )
{
	PROC
	RETURN column -> attribute( name ) -> asInt();
}


int32 *
int32ColumnData(
		const Column *		column )
{
	PROC
	RETURN column -> data() -> int32Data();
}


int32
int32DataSetAttribute(
		const DataSet *		dataSet,
		const char *		name )
{
	PROC
	RETURN dataSet -> attribute( name ) -> asInt();

}


int32
int32TableAttribute(
		const Table *		table,
		const char *		name )
{
	PROC
	RETURN table -> attribute( name ) -> asInt();
}


void
insertRows(
		Table *				table,
		unsigned int			pos,
		unsigned int			count )
{
	PROC
	table -> insertRows( pos, count );
	ENDPROC
}


int
intNullValueArray(
		const Array *			array )
{
	PROC
	RETURN array -> intNullValue();
}


int
intNullValueColumn(
		const Column *			column )
{
	PROC
	RETURN column -> intNullValue();
}


int
intNullValueDataComponent(
		const DataComponent *		dataComponent )
{
	PROC
	RETURN dataComponent -> intNullValue();
}


int
intNullValueNullable(
		const Nullable *		nullable )
{
	PROC
	RETURN nullable -> intNullValue();
}


bool8
isNullArray(
		const Array *			array,
		unsigned int			pos )
{
	PROC
	ArrayData * ad = array -> data();
	const int isNull = ad -> isNull( pos );
	delete ad;
	RETURN isNull;
}


bool8
isNullCell(
		const Column *			column,
		unsigned int			row,
		unsigned int			pos )
{
	PROC
	ColumnData *	cd = column -> data( row, 1 );
	const int isNull = cd -> isNull( pos );
	delete cd;
	RETURN isNull;
}


bool8
isNullColumn(
		const Column *			column,
		unsigned int			row )
{
	PROC
	ColumnData *	cd = column -> data( row, 1 );
	const int isNull = cd -> isNull( 0 );
	delete cd;
	RETURN isNull;
}



void
keepDataSet(
	const char *		dataSetName )
{
	PROC
	dataSetServer -> keep( dataSetName );
	ENDPROC
}


Dal::AccessMode
mode(
		const DataSet *		dataSet )
{
	PROC
	RETURN dataSet -> mode();
}


const char *
nameOfArray(
		const Array * 			array )
{
	PROC
	RETURN array -> name().c_str();
}


const char *
nameOfAttributable(
		const Attributable *		attributable )
{
	PROC
	RETURN attributable -> name().c_str();
}


const char *
nameOfAttribute(
		const Attribute *		attribute )
{
	PROC
	RETURN attribute -> name().c_str();
}


const char *
nameOfBlock(
		const Block *			block )
{
	PROC
	RETURN block -> name().c_str();
}


const char *
nameOfColumn(
		const Column * 			column )
{
	PROC
	RETURN column -> name().c_str();
}


const char *
nameOfDataSet(
		const DataSet * dataSet )
{
	PROC
	RETURN dataSet -> name().c_str();
}


const char *
nameOfTable(
		const Table * 			table )
{
	PROC
	RETURN table -> name().c_str();
}


bool8
nullDefinedArray(
		const Array *			array )
{
	PROC
	RETURN array -> nullDefined();
}


bool8
nullDefinedColumn(
		const Column *			column )
{
	PROC
	RETURN column -> nullDefined();
}


bool8
nullDefinedDataComponent(
		const DataComponent *		dataComponent )
{
	PROC
	RETURN dataComponent -> nullDefined();
}


bool8
nullDefinedNullable(
		const Nullable *		nullable )
{
	PROC
	RETURN nullable -> nullDefined();
}


Nullable::NullType
nullTypeArray(
		const Array *			array )
{
	PROC
	RETURN array -> nullType();
}


Nullable::NullType
nullTypeColumn(
		const Column *			column )
{
	PROC
	RETURN column -> nullType();
}


Nullable::NullType
nullTypeDataComponent(
		const DataComponent *		dataComponent )
{
	PROC
	RETURN dataComponent -> nullType();
}


Nullable::NullType
nullTypeNullable(
		const Nullable *		nullable )
{
	PROC
	RETURN nullable -> nullType();
}


unsigned int
numberOfAttributesInArray(
		const Array *			array )
{
	PROC
	RETURN array -> attributes();
}


unsigned int
numberOfAttributesInAttributable(
		const Attributable *		attributable )
{
	PROC
	RETURN attributable -> attributes();
}


unsigned int
numberOfAttributesInBlock(
		const Block *			block )
{
	PROC
	RETURN block -> attributes();
}


unsigned int
numberOfAttributesInColumn(
		const Column *			column )
{
	PROC
	RETURN column -> attributes();
}


unsigned int
numberOfAttributesInDataSet(
		const DataSet *			dataSet )
{
	PROC
	RETURN dataSet -> attributes();
}


unsigned int
numberOfAttributesInTable(
		const Table *			table )
{
	PROC
	RETURN table -> attributes();
}


unsigned int
numberOfBlocks(
		const DataSet *			dataSet )
{
	PROC
	RETURN dataSet -> blocks();
}


unsigned int
numberOfColumns(
		const Table *			table )
{
	PROC
	RETURN table -> columns();
}


unsigned int
numberOfCommentsOfArray(
      const Array *			array )
{
  	PROC
	RETURN array -> comments() ;
}


unsigned int
numberOfCommentsOfAttributable(
		Attributable * 			attributable )
{
  	PROC
	RETURN attributable -> comments() ;
}


unsigned int 
numberOfCommentsOfBlock(
		Block * 			block )
{
  	PROC
	RETURN block -> comments() ;
}


unsigned int
numberOfCommentsOfDataSet( 
		DataSet *			dataSet )
{
  	PROC
	RETURN dataSet -> comments() ;
}


unsigned int
numberOfCommentsOfTable(
		Table * 			table )
{
  	PROC
	RETURN table -> comments() ;
}


unsigned int
numberOfDimensionsOfArray(
		const Array *			array )
{
	PROC
	RETURN array -> dimensions();
}


unsigned int
numberOfDimensionsOfColumn(
		const Column *			column )
{
	PROC
	RETURN column -> dimensions();
}


unsigned long
numberOfElementsOfArray(
		const Array *			array )
{
	PROC
	RETURN array -> elements();
}


unsigned long
numberOfElementsOfColumn(
		const Column *			column )
{
	PROC
	RETURN column -> elements();
}


unsigned int
numberOfHistorysOfArray(
      Array *			array )
{
	PROC
	RETURN array -> historys() ;
}


unsigned int
numberOfHistorysOfBlock(
      Block *			block )
{
	PROC
	RETURN block -> historys() ;
}


unsigned int
numberOfHistorysOfDataSet(
      DataSet *			dataSet )
{
	PROC
	RETURN dataSet -> historys() ;
}


unsigned int
numberOfHistorysOfTable(
      Table *			table )
{
	PROC
	RETURN table -> historys() ;
}


unsigned long
numberOfRowsOfColumn(
		const Column *			column )
{
	PROC
	RETURN column -> rows();
}


unsigned long
numberOfRowsOfTable(
		const Table *			table )
{
	PROC
	RETURN table -> rows();
}


real32
real32ArrayAttribute(
		const Array *		array,
		const char *		name )
{
	PROC
	RETURN array -> attribute( name ) -> asReal();
}


real32 *
real32ArrayData(
		const Array *		array )
{
	PROC
	RETURN array -> data() -> real32Data();
}


real32
real32Attribute(
		const Attribute *	attribute )
{
	PROC
	RETURN attribute -> asReal();
}


real32
real32BlockAttribute(
		const Block *		block,
		const char *		name )
{
	PROC
	RETURN block -> attribute( name ) -> asReal();
}


real32
real32ColumnAttribute(
		const Column *		column,
		const char *		name )
{
	PROC
	RETURN column -> attribute( name ) -> asReal();
}


real32 *
real32ColumnData(
		const Column *		column )
{
	PROC
	RETURN column -> data() -> real32Data();
}


real32
real32DataSetAttribute(
		const DataSet *		dataSet,
		const char *		name )
{
	PROC
	RETURN dataSet -> attribute( name ) -> asReal();

}


real32
real32TableAttribute(
		const Table *		table,
		const char *		name )
{
	PROC
	RETURN table -> attribute( name ) -> asReal();
}


real64
real64ArrayAttribute(
		const Array *		array,
		const char *		name )
{
	PROC
	RETURN array -> attribute( name ) -> asReal();
}


real64 *
real64ArrayData(
		const Array *		array )
{
	PROC
	RETURN array -> data() -> real64Data();
}


real64
real64Attribute(
		const Attribute *	attribute )
{
	PROC
	RETURN attribute -> asReal();
}


real64
real64BlockAttribute(
		const Block *		block,
		const char *		name )
{
	PROC
	RETURN block -> attribute( name ) -> asReal();
}


real64
real64ColumnAttribute(
		const Column *		column,
		const char *		name )
{
	PROC
	RETURN column -> attribute( name ) -> asReal();
}


real64 *
real64ColumnData(
		const Column *		column )
{
	PROC
	RETURN column -> data() -> real64Data();
}


real64
real64DataSetAttribute(
		const DataSet *		dataSet,
		const char *		name )
{
	PROC
	RETURN dataSet -> attribute( name ) -> asReal();

}


real64
real64TableAttribute(
		const Table *		table,
		const char *		name )
{
	PROC
	RETURN table -> attribute( name ) -> asReal();
}


void
relabelArray(
		Array *		array,
		const char *	newLabel )
{
	PROC
	array -> relabel( newLabel );
	ENDPROC
}


void
relabelAttribute(
		Attribute *	attribute,
		const char *	newLabel )
{
	PROC
	attribute -> relabel( newLabel );
	ENDPROC
}


void
relabelBlock(
		Block *		block,
		const char *	newLabel )
{
	PROC
	block -> relabel( newLabel );
	ENDPROC
}


void
relabelColumn(
		Column *	column,
		const char *	newLabel )
{
	PROC
	column -> relabel( newLabel );
	ENDPROC
}


void
relabelDataSet(
		DataSet *	dataSet,
		const char *	newLabel )
{
	PROC
	dataSet -> relabel( newLabel );
	ENDPROC
}


void
relabelTable(
		Table *		table,
		const char *	newLabel )
{
	PROC
	table -> relabel( newLabel );
	ENDPROC
}


void
releaseArray(
		Array *	array )
{
	PROC
	array -> release();
	ENDPROC
}


void
releaseBlock(
		Block *		/*block*/ )
{
	PROC
	//block -> parent() -> deleteBlock( block -> name() );
	ENDPROC
}


void
releaseColumn(
		Column *	column )
{
	PROC
	column -> release();
	ENDPROC
}


void
releaseDataSet(
		DataSet *	dataSet )
{
	PROC
	dataSetServer -> close( dataSet );
	ENDPROC
}


void
releaseTable(
		Table *		/*table */)
{
	PROC
	//table -> parent() -> deleteBlock( table -> name() );
	ENDPROC
}


void
renameArray(
		Array *		array,
		const char *	newName )
{
	PROC
	array -> rename( newName );
	ENDPROC
}


void
renameAttribute(
		Attribute *	attribute,
		const char *	newName )
{
	PROC
	attribute -> rename( newName );
	ENDPROC
}


void
renameBlock(
		Block *		block,
		const char *	newName )
{
	PROC
	block -> rename( newName );
	ENDPROC
}

void
renameColumn(
		Column *	column,
		const char *	newName )
{
	PROC
	column -> rename( newName );
	ENDPROC
}


void
renameDataSet(
		DataSet *	dataSet,
		const char *	newName )
{
	PROC
	dataSet -> rename( newName );
	ENDPROC
}


void
renameTable(
		Table *		table,
		const char *	newName )
{
	PROC
	table -> rename( newName );
	ENDPROC
}


Attribute*
addAttribute(
		Attributable*			attributable,
		const char*			name,
		const char*			comment,
                const char*			units )
{
	PROC

	Attribute* attribute = attributable -> addAttribute(
						name,
						comment,
						units );

	RETURN attribute;
}


Attribute*
addIntAttribute(
		Attributable*	attributable,
                const char*	name,
		int		value,
		const char*	comment,
                const char*	units )
{
	PROC

	Attribute* attribute = attributable -> addAttribute(
						name,
						comment,
						units );

	*attribute = value;

	RETURN attribute;
}


Attribute*
addRealAttribute(
		Attributable*	attributable,
                const char*	name,
		double		value,
		const char*	comment,
                const char*	units )
{
	PROC

	Attribute* attribute = attributable -> addAttribute(
						name,
						comment,
						units );

	*attribute = value;

	RETURN attribute;
}


Attribute*
addStringAttribute(
		Attributable*	attributable,
                const char*	name,
		const char*	value,
		const char*	comment )
{
	PROC

	Attribute* attribute = attributable -> addAttribute(
						name,
						comment );

	*attribute = string( value );

	RETURN attribute;
}


Attribute*
addBoolAttribute(
		Attributable*	attributable,
                const char*	name,
		bool		value,
		const char*	comment )
{
	PROC

	Attribute* attribute = attributable -> addAttribute(
						name,
						comment );

	*attribute = value;

	RETURN attribute;
}


void
setBoolAttributeValue(
		Attribute*	attribute,
                bool		value )
{
	PROC
	*attribute = value;
	ENDPROC
}


void
setIntAttributeValue(
		Attribute*	attribute,
                int		value )
{
	PROC
	*attribute = value;
	ENDPROC
}


void
setRealAttributeValue(
		Attribute*	attribute,
                double		value )
{
	PROC
	*attribute = value;
	ENDPROC
}


void
setStringAttributeValue(
		Attribute*	attribute,
                const char*	value )
{
	PROC
	*attribute = value;
	ENDPROC
}


void
setBoolArrayAttribute(
		Array *				array,
		const char *			name,
		bool				value,
		const char *			comment )
{
	PROC

	*array -> addAttribute(
			name,
			comment ) = value;

	ENDPROC
}


void
setBoolAttribute(
		Attribute *			attribute,
		bool				value,
		const char *			comment )
{
	PROC

	*attribute -> parent() -> addAttribute(
					attribute -> name(),
					comment ) = value;

	ENDPROC
}


void
setBoolBlockAttribute(
		Block *				block,
		const char *			name,
		bool				value,
		const char *			comment )
{
	PROC

	*block -> addAttribute(
			name,
			comment ) = value;

	ENDPROC
}


void
setBoolColumnAttribute(
		Column *			column,
		const char *			name,
		bool				value,
		const char *			comment )
{
	PROC

	*column -> addAttribute(
			name,
			comment ) = value;

	ENDPROC
}


void
setBoolDataSetAttribute(
		DataSet *			dataSet,
		const char *			name,
		bool				value,
		const char *			comment )
{
	PROC

	*dataSet -> addAttribute(
			name,
			comment ) = value;

	ENDPROC
}


void
setBoolTableAttribute(
		Table *				table,
		const char *			name,
		bool				value,
		const char *			comment )
{
	PROC

	*table -> addAttribute(
			name,
			comment ) = value;

	ENDPROC
}


void
setCellSize(
		Column *			column,
		int				rowNumber,
		int				size )
{
	PROC
	CellData * cellData = column -> cellData( rowNumber );
	cellData -> size( size );
	delete cellData;
	ENDPROC
}


bool
setExists(
		const char *			setName )
{
	PROC
	RETURN dataSetServer -> exists( setName );
}


void
setInt8ArrayAttribute(
		Array *				array,
		const char *			name,
		int8				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*array -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt8Attribute(
		Attribute *			attribute,
		int8				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*attribute -> parent() -> addAttribute(
					attribute -> name(),
					comment,
					units ) = value;

	ENDPROC
}


void
setInt8BlockAttribute(
		Block *				block,
		const char *			name,
		int8				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*block -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt8ColumnAttribute(
		Column *			column,
		const char *			name,
		int8				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*column -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt8DataSetAttribute(
		DataSet *			dataSet,
		const char *			name,
		int8				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*dataSet -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt8TableAttribute(
		Table *				table,
		const char *			name,
		int8				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*table -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt16ArrayAttribute(
		Array *				array,
		const char *			name,
		int16				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*array -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt16Attribute(
		Attribute *			attribute,
		int16				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*attribute -> parent() -> addAttribute(
					attribute -> name(),
					comment,
					units ) = value;

	ENDPROC
}


void
setInt16BlockAttribute(
		Block *				block,
		const char *			name,
		int16				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*block -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt16ColumnAttribute(
		Column *			column,
		const char *			name,
		int16				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*column -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt16DataSetAttribute(
		DataSet *			dataSet,
		const char *			name,
		int16				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*dataSet -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt16TableAttribute(
		Table *				table,
		const char *			name,
		int16				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*table -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt32ArrayAttribute(
		Array *				array,
		const char *			name,
		int32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*array -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt32AttributableAttribute(
		Attributable *			attributable,
		const char *			name,
		int32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*attributable -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt32Attribute(
		Attribute *			attribute,
		int32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*attribute -> parent() -> addAttribute(
					attribute -> name(),
					comment,
					units ) = value;

	ENDPROC
}


void
setInt32BlockAttribute(
		Block *				block,
		const char *			name,
		int32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*block -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt32ColumnAttribute(
		Column *			column,
		const char *			name,
		int32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*column -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt32DataSetAttribute(
		DataSet *			dataSet,
		const char *			name,
		int32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*dataSet -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setInt32TableAttribute(
		Table *				table,
		const char *			name,
		int32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*table -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setNullValueArray(
		Array *				array,
		int				value )
{
	PROC
	array -> nullValue( value );
	ENDPROC
}


void
setNullValueColumn(
		Column *			column,
		int				value )
{
	PROC
	column -> nullValue( value );
	ENDPROC
}

void
setNullValueDataComponent(
		DataComponent *			dataComponent,
		int				value )
{
	PROC
	dataComponent -> nullValue( value );
	ENDPROC
}

void
setNullValueNullable(
		Nullable *			nullable,
		int				value )
{
	PROC
	nullable -> nullValue( value );
	ENDPROC
}


void
setReal32ArrayAttribute(
		Array *				array,
		const char *			name,
		real32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*array -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setReal32Attribute(
		Attribute *			attribute,
		real32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*attribute -> parent() -> addAttribute(
					attribute -> name(),
					comment,
					units ) = value;

	ENDPROC
}


void
setReal32BlockAttribute(
		Block *				block,
		const char *			name,
		real32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*block -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setReal32ColumnAttribute(
		Column *			column,
		const char *			name,
		real32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*column -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setReal32DataSetAttribute(
		DataSet *			dataSet,
		const char *			name,
		real32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*dataSet -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setReal32TableAttribute(
		Table *				table,
		const char *			name,
		real32				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*table -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setReal64ArrayAttribute(
		Array *				array,
		const char *			name,
		real64				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*array -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setReal64Attribute(
		Attribute *			attribute,
		real64				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*attribute -> parent() -> addAttribute(
					attribute -> name(),
					comment,
					units ) = value;

	ENDPROC
}


void
setReal64BlockAttribute(
		Block *				block,
		const char *			name,
		real64				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*block -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setReal64ColumnAttribute(
		Column *			column,
		const char *			name,
		real64				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*column -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setReal64DataSetAttribute(
		DataSet *			dataSet,
		const char *			name,
		real64				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*dataSet -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setReal64TableAttribute(
		Table *				table,
		const char *			name,
		real64				value,
		const char *			units,
		const char *			comment )
{
	PROC

	*table -> addAttribute(
			name,
			comment,
			units ) = value;

	ENDPROC
}


void
setStringArrayAttribute(
		Array *				array,
		const char *			name,
		char *				value,
		const char *			comment )
{
	PROC

	*array -> addAttribute(
			name,
			comment ) = string( value );

	ENDPROC
}


void
setStringAttribute(
		Attribute *			attribute,
		const char *			value,
		const char *			comment )
{
	PROC

	*attribute -> parent() -> addAttribute(
					attribute -> name(),
					comment ) = string( value );

	ENDPROC
}


void
setStringBlockAttribute(
		Block *				block,
		const char *			name,
		char *				value,
		const char *			comment )
{
	PROC

	*block -> addAttribute(
			name,
			comment ) = string( value );

	ENDPROC
}


void
setStringColumnAttribute(
		Column *			column,
		const char *			name,
		char *				value,
		const char *			comment )
{
	PROC

	*column -> addAttribute(
			name,
			comment ) = string( value );

	ENDPROC
}


void
setStringDataSetAttribute(
		DataSet *			dataSet,
		const char *			name,
		char *				value,
		const char *			comment )
{
	PROC

	*dataSet -> addAttribute(
			name,
			comment ) = string( value );

	ENDPROC
}


void
setStringTableAttribute(
		Table *				table,
		const char *			name,
		const char *			value,
		const char *			comment )
{
	PROC

	*table -> addAttribute(
			name,
			comment ) = string( value );

	ENDPROC
}


void
setToNullArray(
		Array *				array,
		unsigned int			pos )
{
	PROC
	ArrayData * ad = array -> data();
	ad -> setToNull( pos );
	delete ad;
	ENDPROC
}


void
setToNullCell(
		Column *			column,
		unsigned int			row,
		unsigned int			pos )
{
	PROC
	ColumnData * cd = column -> data( row, 1 );
	cd -> setToNull( pos );
	delete cd;
	ENDPROC
}


void
setToNullColumn(
		Column *			column,
		unsigned int			row )
{
	PROC
	ColumnData * cd = column -> data( row, 1 );
	cd -> setToNull( 0 );
	delete cd;
	ENDPROC
}




const char *
stringArrayAttribute(
	const Array *			array,
	const char *			name )
{
	PROC
	RETURN array -> attribute( name ) -> asString().c_str();

}


dstring *
stringArrayData(
	const Array *			array )
{
	PROC
	RETURN (dstring *) array -> data() -> stringData();
}


const char *
stringAttribute(
		const Attribute *	attribute )
{
	PROC
	RETURN attribute -> asString().c_str();
}


const char *
stringBlockAttribute(
	const Block *			block,
	const char *			name )
{
	PROC
	RETURN block -> attribute( name ) -> asString().c_str();

}


const char *
stringColumnAttribute(
	const Column *			column,
	const char *			name )
{
	PROC
	RETURN column -> attribute( name ) -> asString().c_str();

}


dstring *
stringColumnData(
	const Column *			column )
{
	PROC
	RETURN (dstring *) column -> data() -> stringData();
}


const char *
stringDataSetAttribute(
	const DataSet *			dataSet,
	const char *			name )
{
	PROC
	RETURN dataSet -> attribute( name ) -> asString().c_str();

}


const char *
stringTableAttribute(
	const Table *			table,
	const char *			name )
{
	PROC
	RETURN table -> attribute( name ) -> asString().c_str();

}


const char *
tableAttributeComment(
		const Table *			table,
		const char *			name )
{
	PROC
	RETURN table -> attribute( name ) -> label().c_str();
}


Attribute::DataType
tableAttributeDataTypeWithName(
		const Table *			table,
		const char *			name )
{
	PROC
	RETURN table -> attribute( name ) -> dataType();
}


Attribute::DataType
tableAttributeDataTypeWithNumber(
		const Table *			table,
		unsigned int			number )
{
	PROC
	RETURN table -> attribute( number ) -> dataType();
}



const char *
tableAttributeUnits(
	const Table *			table,
	const char *			name )
{
	PROC
	RETURN table -> attribute( name ) -> units().c_str();
}


Attribute *
tableAttributeWithName(
		Table *				table,
		const char *			name )
{
	PROC
	RETURN table -> attribute( name );
}


Attribute *
tableAttributeWithNumber(
		Table *				table,
		unsigned int			number )
{
	PROC
	RETURN table -> attribute( number );
}


bool8
tableHasAttribute(
	const Table *			table,
	const char *			name )
{
	PROC
	RETURN table -> hasAttribute( name );
}


bool8
tableHasColumn(
	       const Table * table,
	       const char * name)
{
	PROC
	RETURN table -> hasColumn( name );
}


const char *
tableLabel(
	const Table *			table )
{
	PROC
	RETURN table -> label().c_str();
}


Attributable *
tableToAttributable(
		Table *				table )
{
	PROC
	RETURN table;
}


Block *
tableToBlock(
	Table *			table )
{
	PROC
	RETURN table;
}


Table *
tableWithName(
	const DataSet *			dataSet,
	const char *			name )
{
	PROC
	RETURN const_cast<Table *>( dataSet -> table( name ));
}


Table *
tableWithNumber(
	const DataSet *			dataSet,
	unsigned int			position )
{
	PROC
	RETURN const_cast<Table *>( dataSet -> table( position ));
}


//int16 *
//int16CellData( Column * column, unsigned long rowNumber )
//{
//	PROC
//	const Data * cd = column -> cellData( rowNumber );
//	RETURN cd -> int16Data();
//}


//int32 *
//int32CellData( Column * column, unsigned long rowNumber )
//{
//	PROC
//	const Data * cd = column -> cellData( rowNumber );
//	RETURN cd -> int32Data();
//}


uint16 *
uint16ArrayData(
		const Array *			array )
{
	PROC
	RETURN array -> data() -> uint16Data();
}


uint16 *
uint16CellData(
		const Column *			column,
		unsigned long			rowNumber )
{
	PROC
	const Data * cd = column -> cellData( rowNumber );
	RETURN cd -> uint16Data();
}


uint16 *
uint16ColumnData(
		const Column *			column )
{
	PROC
	RETURN column -> data() -> uint16Data();
}


uint32 *
uint32ArrayData(
		const Array *			array )
{
	PROC
	RETURN array -> data() -> uint32Data();
}


uint32 *
uint32CellData(
		const Column *			column,
		unsigned long			rowNumber )
{
	const Data * cd = column -> cellData( rowNumber );
	RETURN cd -> uint32Data();
}


uint32 *
uint32ColumnData(
		const Column *			column )
{
	PROC
	RETURN column -> data() -> uint32Data();
}


}



template<class T, T * (Data::*d)() const >
T * cellData( Column * column, unsigned long rowNumber )
{
	PROC
	const Data * cd = column -> cellData( rowNumber );
	RETURN (cd ->* d)();
}


extern "C"
{

// **********************************************************************
// We would like to be able to declare the cell data access functions by
// pointer, rather than by actual implementation, but this gives
// rise to a run-time fatal error (illegal instruction). So the following
// line is commented out for the moment.
// **********************************************************************
//#define celldata(type,fn) type*(*fn##CellData)( Column *,unsigned long) = cellData<type,&CellData::fn##Data>
#define celldata(type,fn) type* fn##CellData( Column * column,unsigned long rowNumber) { PROC RETURN cellData<type,&CellData::fn##Data>( column, rowNumber ); }

celldata(int8,int8);
celldata(int16,int16);
celldata(int32,int32);
celldata(real32,real32);
celldata(real64,real64);
celldata(bool8,bool);
celldata(char,string);


#define toattributable(fn,tp) Attributable* fn##ToAttributable( tp * attributable ) { PROC RETURN attributable; }

//toattributable(array,Array);
//toattributable(block,Block);
//toattributable(column,Column);
//toattributable(dataSet,DataSet);
//toattributable(table,Table);

}


