! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! This example illustrates the use of the dataComponent() function.
! The units of objects with data type BOOLEAN and STRING are meaningless
! and so are not displayed.
subroutine displayUnits( dcomponent )
  use dal
 
  implicit none

  type(DataComponentT) dcomponent
  integer dattype

  dattype = dataType( dcomponent )
  write(*,*) dattype
  if(dattype.eq.INTEGER8.or.dattype.eq.INTEGER16.or.dattype.eq.INTEGER32 &
    .or.dattype.eq.REAL32.or.dattype.eq.REAL64) then
    write(*,*) units( dcomponent )
  end if

end subroutine displayUnits

program example_datacomponent

  use dal
  
  implicit none

  type(ArrayT) arr
  type(BlockT) blk
  type(ColumnT) col
  type(DataSetT) set
  type(TableT) tab
  integer i, j
  integer, dimension(3), parameter :: s = (/ 2,3,4 /)

  set = dataSet("test.dat",CREATE)
  tab = addTable(set,"some table",100)

  col = addColumn(tab,"bool",BOOLEAN)
  col = addColumn(tab,"int8",INTEGER8,units="cm",comment="int8 column")
  col = addColumn(tab,"int16",INTEGER16,units="dm",comment="int16 column")
  col = addColumn(tab,"int32",INTEGER32,units="m",comment="in32 column")
  col = addColumn(tab,"real32",REAL32,units="Dm",comment="real32 column")
  col = addColumn(tab,"real64",REAL64,units="hm",comment="real64 column")
  col = addColumn(tab,"string",STRING,comment="string column",dimensions=(/80/))
  arr = addArray(set, "array1", INTEGER16, dimensions=s, units="klm" )
  arr = addArray(set, "array2", INTEGER32, dimensions=s, units="kla" )

  do i = 0, numberOfBlocks( set ) - 1
    blk = block( set, i, READ )
    if( blockType( blk ).eq.ARRAY_BLOCK ) then
      arr = array( set, name( blk ), READ )
      call displayUnits( dataComponent( arr ) )
    else
      tab = table( set, name( blk ) )
      do j = 0, numberOfColumns( tab ) - 1
        col = column( tab, j, READ )
        call displayUnits( dataComponent( col ) )
      end do
    end if
  end do
  call release(set)

end program example_datacomponent
