! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! This examples show how the forEachColumn() function is used.
! The column by name is used to get a column and rename it.
! The column by number is used to iterate over all
! columns in the table to output the name, type and units.
subroutine displayColumn( col )

  use dal

  implicit none

  type(ColumnT), intent(in) :: col

  write(*,*) name( col ), columnDataType( col ), units( col )

end subroutine displayColumn

program example_foreachcolumn

  use dal
  
  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col
  integer i

  interface
    subroutine displayColumn( col )
      use dal
      implicit none

      type(ColumnT), intent(in) :: col
    end subroutine displayColumn
  end interface

  set = dataSet("test.dat",CREATE)
  tab = addTable(set,"some table",100)
  col = addColumn(tab,"col1",INTEGER32,units="m1",comment="in32 column")
  col = addColumn(tab,"col2",INTEGER32,units="m2",comment="in32 column")
  col = addColumn(tab,"col3",INTEGER32,units="m3",comment="in32 column")

  col = column( tab, "col2", MODIFY )
  call rename( col, "col4" )

  call forEachColumn( tab, displayColumn )
  call release(set)

end program example_foreachcolumn
