! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! This example shows how to use the int64Array2Data interface.
! In the example a dataset is created (opened) containing
! a table with 2 vector arrays.
!
! The second array has the same data type as the first; this
! is ensured by using the arrayDataType() function to determine
! the data type of the first array.
!
! The array is then initialised,
program example_arrayvectordata

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ArrayT) arr1, arr2
  real(kind=DOUBLE), dimension(:), pointer :: a1, a2
  integer, dimension(1), parameter :: s = (/ 3 /)
  integer :: i,n

  ! create a set
  set = dataSet("test.dat",CREATE)
  arr1 = addArray( set, "array1", REAL64, s, "km", "array comment" )
  arr2 = addArray( set, "array2", REAL64, s, "km", "array comment" )

  ! fill with unique numbers

  n = 0
  a1 => real64VectorData(arr1)
  a2 => real64VectorData(arr2)
  do i=0,2 
    a1(i) = n
    a2(i) = a1(i)
    n = n + 1
  end do

  call release(arr1)
  call release(arr2)
  call release(set)

end program example_arrayvectordata
