// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include <Dal.h>
#include <iostream>
#include <cmath>

class Report : public TableProcessor {

  void table(Table*) {
    xData = addColumn("x",Column::Read);
    yData = addColumn("y",Column::Read);
    rData = addColumn("r",Column::Create);
  }

  void proc(SubTable& st) {
    // One could use st.data("x") instead of xData
    int16  *x = xData->int16Data();
    int16  *y = yData->int16Data();
    real64 *r = rData->real64Data();

    for (unsigned i = 0; i < st.count(); ++i) {
      clog << x[i] << ' ' << y[i] << endl;
      r[i] = sqrt( sqr(x[i]) + sqr(y[i]) );
    }
  }

  ColumnData *xData, *yData, *rData;
};

int main()
{
  DataSet *set = dataSetServer->open("test.dat",
	DataSet::Modify);
  Table* table = set->table("events",Table::Modify);
  Report report;
  report.process(table);
  delete set;
  return 0;
}
