// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*************************************************************** 
* 
* DalTable.h & XMM Science Analysis System (SAS) 
* 
* Class representing DAL Table. 
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: DalTable.h,v 1.2 2004/03/05 14:43:01 eojero Exp $
* 
*****************************************************************/ 
#ifndef DAL_TABLE_H
#define DAL_TABLE_H


#include <Dal.h>


#include "DalBlock.h"
#include "DalDataSet.h"
#include "DalSeekable.h"


class DalColumn;


class DalTable : public virtual Table, public DalBlock, public DalSeekable<unsigned long>
{
	public :

		DalTable(
			const std::string &		name,
			const std::string &		label,
			unsigned long		rows,
			DalDataSet *		parent );

		virtual ~DalTable(){}
		virtual DalTable& operator=( const Table& ) = 0;

		Column * addColumn(
				const std::string &			name,
				Data::DataType			dataType,
				const std::string &			label,
				const std::string &			units,
				const longVec &			size,
				int				position = -1 );

		Column * add(
				const Column *			column,
				const std::string& 			newName = "",
				int				position = -1 );

		unsigned long rows() const;

		Column * column(
			unsigned int	columnNumber,
			AccessMode	accessMode=AsParent );

		const Column * column(
			unsigned int	columnNumber,
			AccessMode	accessMode=AsParent ) const;

		Column * column(
			const std::string &	columnName,
			AccessMode	accessMode=AsParent );

		const Column * column(
			const std::string &	columnName,
			AccessMode	accessMode=AsParent ) const;

		bool hasColumn( const std::string & name ) const;
		void deleteColumn( unsigned int columnNumber );
		void deleteColumn( const std::string & columnName );
		unsigned int columns() const;
		unsigned int columnNumber( const std::string & columnName ) const;
		void copyRows( unsigned long from, unsigned long to, unsigned long count );
		void insertRows( unsigned long pos, unsigned long count );
		void deleteRows( unsigned long from, unsigned long count );
		//void releaseAll();
		void printOn( ostream & os ) const;
		void release();
		void seek( unsigned long from, unsigned long count );

	protected :

		unsigned long 		_rows;

		typedef vector<DalColumn*> ColumnList;

		ColumnList		_columns;

		void			insert( DalColumn * column, int position );

		void checkValidColumnNumber( unsigned int columnNumber ) const;

		virtual DalColumn * makeColumn(
				const std::string &			name,
				Data::DataType			dataType,
				const std::string &			label,
				const std::string &			units,
				const longVec &			size,
				DalTable *			parent ) = 0;

};


#endif


