// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//  Giuseppe Vacanti (cosine science & computing bv)
//  March  1, 2003
// 
//   $Id: SmartDalImplementation.cc,v 1.3 2009/06/22 15:58:03 rsaxton Exp $

#include<SmartDal>
/*AI gcc 4 porting 
strncpy was not declared in this scope
*/
#include<string.h>
// The static_cast is the price to pay for convenience.
SmartDal::DataSetH SmartDal::open_set(const std::string & name,
				      AccessMode mode, MemoryModel model) {
    // FXIME: the int cast is necessary with 2.95.3.
  return dataSetServer->open(name, static_cast<Dal::AccessMode>((int)mode),
			     static_cast<DataSetServer::MemoryModel>((int)model));
}


template<class T> void SmartDal::Deallocator::operator()(T *) {}

template<> void SmartDal::Deallocator::operator()(DataSet * s){
    dataSetServer->close(s);
}	    
    
// Instantiates all the required templates

template void SmartDal::Deallocator::operator()(Attribute *);
template void SmartDal::Deallocator::operator()(Data *);
template void SmartDal::Deallocator::operator()(CellData *);
template void SmartDal::Deallocator::operator()(Array *);
template void SmartDal::Deallocator::operator()(Block *);
template void SmartDal::Deallocator::operator()(ColumnData *);
template void SmartDal::Deallocator::operator()(Column *);
template void SmartDal::Deallocator::operator()(Table *);

std::string SmartDal::get_string_row(const ColumnDataH & col,
				     unsigned long row) {
  unsigned long len = col->size()[0];
  if ((col->stringData() + col->position(row) + len - 1)[0] != '\0') {
    char s[len+1];
    strncpy(s, col->stringData() + col->position(row), len);
    s[len] = '\0';
    return std::string(s);
  }
  return std::string(col->stringData() + col->position(row));
}

void SmartDal::set_string_row(const ColumnDataH & col,
			      unsigned long row, const std::string & content){
  unsigned long len = col->size()[0];
  strncpy(col->stringData() + col->position(row), content.c_str(), len);
}

std::string SmartDal::get_string_row(const ColumnH & col, unsigned long row){
  return get_string_row(col->data(), row);
}

void SmartDal::set_string_row(const ColumnH & col, unsigned long row,
		    const std::string & content){
  set_string_row(col->data(), row, content);
}
