// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#ifndef __DalExt_H
#define __DalExt_H


/*
  The DAL supports from itself only random access into columns.
  The extended DAL adds automatic and manual iteration and random
  access into a selected set of columns of a table.
  See also the DAL documentation, section Iteration.
*/


#include "Dal.h"
#include "Model.h"


class SubTableProcessor;


class SubTable : public virtual Child<Table>, public virtual Child<SubTableProcessor>, public virtual Seekable<unsigned long>
{
	public :
		virtual ~SubTable() {}
		virtual ColumnData * data( const std::string & name ) const = 0;
		virtual ColumnData * data( unsigned int columnNumber ) const = 0;
		virtual CellData * cellData( const std::string & name, unsigned long row ) = 0;
		virtual CellData * cellData( unsigned int columnNumber, unsigned long row ) = 0;
		virtual void initialise() = 0;
	private :

};


class SubTableProcessor
{
	public :

    		SubTableProcessor( Table * table );
		virtual ~SubTableProcessor() {}

		bool next();
		SubTable * subTable() const;
		void forEachSubTable();
		void forEachRow();

	private :

		virtual void proc( SubTable * ) {}

		SubTable *	_subTable;
		Model		_model;
		unsigned long	_from;

};


template<class T>
class BlockIterator
{
	public :
		virtual ~BlockIterator() {}
		virtual T * next() = 0;
		virtual void begin() = 0;
};


class ArrayIterator : public virtual BlockIterator<Array>
{
	public :
		virtual ~ArrayIterator() {}
};


class TableIterator : public virtual BlockIterator<Table>
{
	public :
		virtual ~TableIterator() {}
};


class BlockIteratorServer
{
	public :
		virtual ArrayIterator * arrayIterator( DataSet* dataSet ) const = 0;
		virtual TableIterator * tableIterator( DataSet* dataSet ) const = 0;
};


extern BlockIteratorServer* blockIteratorServer;


#endif
