// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*************************************************************** 
* 
* HiLowArray.h               & XMM Science Analysis System (SAS) 
* 
* Class representing block arrays.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: HiLowArray.h,v 1.2 2004/03/05 14:43:02 eojero Exp $
* 
*****************************************************************/ 
#ifndef HILOW_ARRAY_H
#define HILOW_ARRAY_H


#include <Dal.h>

#include "DalArray.h"
#include "HiLowDataStorage.h"
#include "DalSeekable.h"


class HiLowArray : public DalArray, public DalSeekable< longVec >
{
	public :

		// Constructors
		HiLowArray(
			const std::string &			name,
			DataType 			dataType,
			const longVec &			size,
			const std::string &			units,
			const std::string &			label,
			DalDataSet *			parent );

		HiLowArray(
			const Array & 	original,
			DalDataSet * 	parent,
			std::string 		newName = "");

		virtual ~HiLowArray();

		HiLowArray& operator=( const Array& );
 
		unsigned long elements() const;
		unsigned int dataSize() const;
		//ArrayData * data(
		//		const longVec &		from = zero,
		//		const longVec &		to = zero,
		//		AccessMode		accessMode=AsParent) const;

		void scaleData(
				double			zero,
				double			scale,
				DataType		toType );

		void unscaleData(
				DataType		toType );

		void printOn( ostream & os ) const;

		//void release();

	private :

		void	setData( const HiLowArray &	usingDataInThisArray );

		HiLowDataStorage *		_data;

		ArrayData * makeArrayData(
				AccessMode		accessMode ) const;

};



#endif
