// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*************************************************************** 
* 
* HiLowData.h                  & XMM Science Analysis System (SAS) 
* 
* Class representing data object.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: HiLowData.h,v 1.2 2004/03/05 14:43:02 eojero Exp $
* 
*****************************************************************/ 
#ifndef HILOW_DATA_H
#define HILOW_DATA_H


#include "Dal.h"
#include "HiLowDataStorage.h"
#include "Tracer.h"
#include "DalSeekable.h"
#include "DataManipulator.h"
#include "DalData.h"


template<class T>
class HiLowData : public virtual DataManipulator, public DalData<T>
{
	public :

		// Constructors
		HiLowData(
			HiLowDataStorage *		data,
			unsigned long			from,
			unsigned long			count,
			const T *			    parent,
			Dal::AccessMode			accessMode )
			:
				DalData<T>( parent, from, count, accessMode ),
				_data( data )
		{
			PROC
			ENDPROC
		}
		virtual ~HiLowData(){}
		HiLowDataStorage * dataStorage() const { PROC RETURN _data; }
		unsigned int dataSize() const { PROC RETURN _data -> dataType(); }
		Dal::DataType dataType() const { PROC RETURN _data -> dataType(); }
		void printOn( ostream & os, const std::string & sep="" ) const
		{ PROC _data -> printOn( os, sep ); ENDPROC }



	private :

		HiLowDataStorage *		_data;
};


#endif

