// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*************************************************************** 
* 
* MemColumn.h               & XMM Science Analysis System (SAS) 
* 
* Class representing DAL Table Column.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA, and Marco Beijersbergen
* 
* $Id: MemColumn.h,v 1.2 2004/03/05 14:43:03 eojero Exp $ 
* 
*****************************************************************/ 
#ifndef MEM_COLUMN_H
#define MEM_COLUMN_H


#include <Dal.h>

#include "DalChild.h"
#include "DataStorage.h"
#include "DalColumn.h"
#include "DalSeekable.h"


class MemColumn : public DalColumn, public DalSeekable<unsigned long>
{
	public :

		// Constructors
		MemColumn(
			const std::string &			name,
			DataType			dataType,
			const std::string &			label,
			const std::string &			units,
			const std::vector<unsigned long> &	size,
			DalTable *			parent );

		MemColumn(
			const DalColumn &		original,
			DalTable * 			parent,
			std::string				newName = "" );

		virtual  ~MemColumn();

		DalColumn & operator=( const Column & );

		void			rename( const std::string & newName );
		unsigned long		rows() const { PROC RETURN DalChild<Table>::parent()->rows(); }
		unsigned long		elements() const;
		unsigned int		dataSize() const;

		//ColumnData * data(
		//		unsigned long			from=0,
		//		unsigned long			count=0,
		//		AccessMode			accessMode=AsParent ) const;

		//CellData * cellData(
		//		unsigned long			rowNumber,
		//		unsigned long			from=0,
		//		unsigned long			count=0,
		//		AccessMode			accessMode=AsParent ) const;

		void scaleData(
				double			zero,
				double			scale,
				DataType		toType );

		void unscaleData(
				DataType		toType );


		void printOn( ostream & is ) const;

		void insertRows( unsigned long pos, unsigned long count=1 );

		void deleteRows( unsigned long pos, unsigned long count=1 );

		void copyRows( unsigned long from, unsigned long to, unsigned long count=1 );

		//void release();

	private :

		// Forbid.
		MemColumn(const MemColumn&);

		DataStorage **			_data;

		void	assignFixed( const Column & column );
		void	assignVariable( const Column & column );
		void	setData( const MemColumn &	usingDataInThisColumn );

		CellData * makeCellData(
				unsigned long			rowNumber,
				unsigned long			from,
				unsigned long			count,
				AccessMode			accessMode ) const;

		ColumnData * makeColumnData(
				unsigned long			from,
				unsigned long			count,
				AccessMode			accessMode ) const;


};


#endif


