// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
/*************************************************************** 
* 
* MemTable.cc             & XMM Science Analysis System (SAS) 
* 
* Class representing DAL Dataset Table.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
* 
* $Id: MemTable.cc,v 1.2 2004/03/05 14:43:03 eojero Exp $ 
* 
*****************************************************************/ 


#include <errstr.h>
#include <Tracer.h>
#include <stringutil.h>


#include "MemTable.h"
#include "MemColumn.h"


MemTable::MemTable(
			const string &	name,
			unsigned long	rows,
			const string &	label,
			DalDataSet *	parent )

		:
		DalTable( name, label, rows, parent )
{
	PROC

	ENDPROC

}


//DalTable *
//MemTable::makeTable(
//		const string &	name,
//		unsigned long	rows,
//		const string &	label,
//		DalDataSet *	parent )
//{
//	PROC
//	RETURN new MemTable( name, rows, label, parent );
//}


MemTable::MemTable(
			const Table&	table,
			DalDataSet*	parent,
			const string&	newName)
		:
		DalTable(
			newName.empty() ? table.name() : newName,
			table.label(),
			table.rows(),
			parent)
{
	PROC

	operator=(table);

	ENDPROC
}

MemTable::~MemTable()
{
	PROC

	for (ColumnList::iterator i = _columns.begin(); i != _columns.end(); ++i)
		delete (*i);

	ENDPROC
}


DalTable &
MemTable::operator=( const Table & table )
{
	PROC

	(Attributable&)*this = table;

	for (ColumnList::iterator i = _columns.begin(); i != _columns.end(); ++i)
		delete (*i);

	this -> _rows = table.rows();

	for (unsigned i = 0; i < table.columns(); ++i)
	{
		const Column *org = table.column(i);
		add( org, org -> name() );
		//_columns.push_back( new MemColumn( *org, this ) );
	} // for

	RETURN *this;
}


DalColumn *
MemTable::makeColumn(
		const string &			name,
		Data::DataType			dataType,
		const string &			label,
		const string &			units,
		const longVec &	size,
		DalTable *			parent )
{
	PROC
	RETURN new MemColumn(
				name,
				dataType,
				label,
				units,
				size,
				parent );

}


void
MemTable::forEachSubTable( void(*callThisFunction)( Table * table ))
{
	PROC
	callThisFunction( this );
	ENDPROC
}

