// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.



#ifndef HILOW_FITS_DATABASE_H
#define HILOW_FITS_DATABASE_H

#include <fitsio.h>

#include "Dal.h"
#include "Database.h"


class HiLowFitsDatabase : public virtual Database
{
	public :

		HiLowFitsDatabase();
		virtual ~HiLowFitsDatabase();
		void addEntry( const DataSet & dataSet );
		void addEntry( const Block & block );
		void addEntry( const Table & table );
		void addEntry( const Array & array );
		void addEntry( const Column & column );
		void setVariableColumnSize( const CellData & cellData );
		void deleteEntry( const DataSet & dataSet );
		void deleteEntry( const Block & block );
		void deleteEntry( const Table & table );
		void deleteEntry( const Array & array );
		void deleteEntry( const Column & column );
		bool hasEntry( const DataSet & dataSet );
		bool hasEntry( const Block & block );
		bool hasEntry( const Table & table );
		bool hasEntry( const Array & array );
		bool hasEntry( const Column & column );

		void insertRows(
				const Table &	table,
				unsigned long	pos,
				unsigned long	count );

		void deleteRows(
				const Table &	table,
				unsigned long	from,
				unsigned long	count );

		void copyRows(
				const Column &	column,
				unsigned long	from,
				unsigned long	to,
				unsigned long	count );

		void dumpData(
				const Column & 	column,
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				bool8 *		values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				bool8 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				bool8 *		values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Column & 	column,
				unsigned long	rowNumber,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				bool8 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Column & 	column,
				unsigned long	rowNumber,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Array & 	array, 
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Array & 	array, 
				bool8 *		values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Array & 	array, 
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Array & 	array,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Array & 	array,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Array & 	array,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Array & 	array,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Array & 	array,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void dumpData(
				const Array & 	array,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Array & 	array,
				char ** 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Array & 	array,
				bool8 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Array & 	array,
				int8 * 		values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Array & 	array,
				uint16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Array & 	array,
				int16 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Array & 	array,
				uint32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Array & 	array,
				int32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Array & 	array,
				real32 * 	values,
				unsigned long 	from,
				unsigned long 	count );
		void loadData(
				const Array &	array,
				real64 * 	values,
				unsigned long 	from,
				unsigned long 	count );

	protected :

		fitsfile *	_fptr;

	private :
		enum MoveToMode { Before, To };
		void moveToExtension( const Table & table, MoveToMode moveToMode = To );
		void moveToExtension( const Array & array, MoveToMode moveToMode = To );
		void openFitsFile( const Block& block );
};


#endif

