// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
/*************************************************************** 
* 
* HiLowFitsReaderWriter.h     & XMM Science Analysis System (SAS) 
* 
* Class representing Fits File reader/writer.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
*                and Marco Beijersbergen
* 
* $Id: HiLowFitsReaderWriter.cc,v 1.3 2009/06/22 15:58:03 rsaxton Exp $
* 
*****************************************************************/ 


#include <string>
#include <iostream>
#include <fitsio.h>
#include <unistd.h>
#include <sstream>
/* AI gcc4 porting
   strlen was not declared in this scope
*/
#include <string.h>

#include <stringutil.h>
#include <Tracer.h>
#include <errstr.h>
#include <Dal.h>

#include "utilities.h"
#include "HiLowFitsReaderWriter.h"
#include "dal_utilities.h"


// ********************
// Global defined here.
// ********************

static const string nameOfPrimaryImage = "PRIMARY";
static const string defaultArrayComment = "The name of this array";
static const string defaultTableComment = "The name of this table";
static const string defaultColumnUnitsComment = "Units of column";

/*
int
HiLowFitsReaderWriter::deleteCommentsAndHistory(
	fitsfile *		fptr,
	KeywordChecker *	attributeChecker,
	int &			status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	if( _dataSet -> mode() == Dal::Read )
	{
		ALTRETURNV( 0 );
	} // if


	int			keyNumber;
	char			keyName[ 1024 ];
	char 			comment[ 1024 ];
	char			value[ 1024 ];

	keyNumber = 1;		// The first keyword in a table
				// has position 1.

	attributeChecker -> reset();

	while( !status )
	{
		fits_read_keyn(
				fptr,
				keyNumber,
				keyName,
				value,
				comment,
				&status );

		// ------------------------------------
		// Either, there are no more keywords
		// (KEY_OUT_BOUNDS), in which case,
		// do not attempt to read any more
		// and exit from the loop, otherwise,
		// there was a real error.
		// ------------------------------------
		if (status == KEY_OUT_BOUNDS) {

			status = 0;

			ALTRETURNV( 0 );

		}

		fits_check( fptr );

		if( attributeChecker -> ok( keyName )
		    &&
		    ( string( "COMMENT" ) == keyName || string( "HISTORY" ) == keyName ))
		{
			fits_delete_record( fptr, keyNumber, &status );
			fits_check( fptr );
		}
		else
		{
			keyNumber++;
		} // if
	} // while

	// ********************************
	// If we get here an error occured.
	// ********************************

	RETURN 1;

}
*/


int
HiLowFitsReaderWriter::deleteCommentsAndHistory(
	fitsfile *		fptr,
	int &			status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	if( _dataSet -> mode() == Dal::Read )
	{
		ALTRETURNV( 0 );
	} // if

	int			keyNumber;
	char			keyName[ 1024 ];
	char 			comment[ 1024 ];
	char			value[ 1024 ];

	keyNumber = 1;		// The first keyword in a table
				// has position 1.

	while( !status )
	{
		fits_read_keyn(
				fptr,
				keyNumber,
				keyName,
				value,
				comment,
				&status );

		// ------------------------------------
		// Either, there are no more keywords
		// (KEY_OUT_BOUNDS), in which case,
		// do not attempt to read any more
		// and exit from the loop, otherwise,
		// there was a real error.
		// ------------------------------------
		if (status == KEY_OUT_BOUNDS)
		{

			status = 0;

			ALTRETURNV( 0 );

		} // if

		fits_check( fptr );

		if( string( "COMMENT" ) == keyName || string( "HISTORY" ) == keyName )
		{
			fits_delete_record( fptr, keyNumber, &status );
			fits_check( fptr );
		}
		else
		{
			keyNumber++;
		} // if
	} // while

	// ********************************
	// If we get here an error occured.
	// ********************************

	RETURN 1;
}
// Method to remove the History records
int
HiLowFitsReaderWriter::deleteHistory(
	fitsfile *		fptr,
	int &			status ) const
{
	PROC

	if( status != 0 )
		ALTRETURNV( 1 );

	if( _dataSet -> mode() == Dal::Read )
		ALTRETURNV( 0 );

	int			keyNumber;
	char			keyName[ 1024 ];
	char 			comment[ 1024 ];
	char			value[ 1024 ];

	keyNumber = 1;		// The first keyword in a table
				// has position 1.

	while( !status )
	{
		fits_read_keyn(
				fptr,
				keyNumber,
				keyName,
				value,
				comment,
				&status );

		// ------------------------------------
		// Either, there are no more keywords
		// (KEY_OUT_BOUNDS), in which case,
		// do not attempt to read any more
		// and exit from the loop, otherwise,
		// there was a real error.
		// ------------------------------------
		if (status == KEY_OUT_BOUNDS)
		{
			status = 0;

			ALTRETURNV( 0 );
		} // if

		fits_check( fptr );

		if( string( "HISTORY" ) == keyName )
		{
			fits_delete_record( fptr, keyNumber, &status );
			fits_check( fptr );
		}
		else
		{
			keyNumber++;
		} // if
	} // while

	// ********************************
	// If we get here an error occured.
	// ********************************

	RETURN 1;
}

// Method to removed unwanted keywords
int
HiLowFitsReaderWriter::deleteUnwantedKeyword(
	fitsfile *	fptr,
	const string&	keyword,
	int &		status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	fits_delete_key(
			fptr,
			(char*)keyword.c_str(),
			&status );

	// ------------------------------------
	// If the keyword was not found
	// this is not an error; we will
	// just reset the status and exit.
	// ------------------------------------
	if( status == KEY_NO_EXIST )
	{
		status = 0;

		ALTRETURNV( 0 );

	} // if

	fits_check(fptr);

	// ********************************
	// If we get here an error occured.
	// ********************************

	RETURN 1;

}


int
HiLowFitsReaderWriter::writeColumns(
	fitsfile *	fptr,
	const Table &	table,
	int &		status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if


	// **********************
	// For each Table Column.
	// **********************
	for( unsigned int j = 0; j < table.columns(); ++j )
	{
		const Column *		column = table.column( j );


		// =============================
		// write Column Scaling Factors.
		// =============================
		writeColumnScalingFactors( fptr, column, j, status );

		// =========================
		// Write the column comment.
		// =========================
		writeColumnComment( fptr, column -> label(), j, status );

		// =================
		// Write column name
		// =================
		fits_update_key_str(
			fptr,
			(char *)("TTYPE" + toString( j + 1 )).c_str(),	// This is how to set the column units. 
			(char *)column -> name().c_str(),
			0,						// Comment unchanged
			&status );
		fits_check(fptr);

		// ==========
		// write tdim
		// ==========
		if( column -> dimensions() > 0 )
		{
			// **************************************************
			// To prevent the tdim keywords from being duplicated
			// it is first necessary to delete the current
			// tdim (if it exists) keyword, manually, since
			// fits does not have a function fits_update_tdim().
			// **************************************************
			fits_delete_key(
				fptr,
				(char *)("TDIM" + toString( j + 1 )).c_str(), // This is how to delete the column dimensions 
				&status );

			if( status == KEY_NO_EXIST ) status = 0;
			fits_check( fptr );

			// **************************************
			// But do not bother writing TDIMs of the
			// form TDIM '(n)'
			// **************************************
			if( column -> dimensions() > 1 )
			//if(
			//	column -> dimensions() != 1
			//	&&
			//	column -> size()[0] != 1 )
			{
				fits_write_tdim(
					fptr,
					j + 1,
					column -> dimensions(),
					(long *)Mvec<unsigned long>( column -> size() ).data(),
					&status );
			} // if
		} // if

		// ============================
		// Write column units (if any).
		// ============================
		if( !column -> units().empty() )
		{
			fits_update_key_str(
				fptr,
				(char *)("TUNIT" + toString( j + 1 )).c_str(),	// This is how to set the column units. 
				(char *)column -> units().c_str(),
				(char *)defaultColumnUnitsComment.c_str(),
				&status );
			fits_check(fptr);
		} // if

		// ========================
		// Write column null value.
		// ========================
		if( column -> nullDefined() )
		{
			writeColumnNullValue( fptr, column, status );
		}
		else
		{
			// =======================================
			// Delete the TNULL keyword (if it exists)
			// =======================================
			const string tnull( "TNULL" + toString( column -> columnNumber() + 1 ));

			deleteUnwantedKeyword( fptr, tnull, status );
		} // if

		// ========================
		// Write Column Attributes.
		// ========================
		writeHeader(
			fptr,
			*column,
			toString( j + 1 ),
			&FitsReaderWriter::columnAttributeOK,
			status );

	} // for

	// ******************************
	// Everything is Ok, so return 0.
	// ******************************

	RETURN 0;

}


int
HiLowFitsReaderWriter::writeTable( 
	fitsfile *	fptr,
	const Table &	table,
	int &		status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	// ***************************************
	// Move to the appropriate fits extension.
	// ***************************************
	int			hdutype;
	//const int		hdunum = table.parent() -> blockNumber( table.name() ) + 2;
	const unsigned int	hdunum = blockNumberToHdunum( table );

	fits_movabs_hdu(
		fptr,
		hdunum,
		&hdutype,
		&status );
	fits_check(fptr);

	deleteAttributes( fptr, status );
        // RDS - removed this command and replaced with one which 
        // only deletes the history (SPR-7608)
	//deleteCommentsAndHistory( _fptr, status );
	deleteHistory( _fptr, status );

	// ***************************************************
	// Set the table name. This is achieved by setting the
	// FITS file keyword EXTNAME.
	// ***************************************************
	fits_update_key_str(
			fptr,
			"EXTNAME",	// This is how to set the name.
			(char *)table.name().c_str(),
			table.label() == "" ? (char*)defaultTableComment.c_str() : (char *)table.label().c_str(),
			&status );

	fits_check(fptr);

	// *******************
	// write table header.
	// *******************
	writeHeader(
		fptr,
		(const Attributable&)table,
		"",
		&FitsReaderWriter::tableAttributeOK,
		status );

	// ************************
	// Write the table columns.
	// ************************
	writeColumns( fptr, table, status );

	// ******************************
	// Everything is Ok, so return 0.
	// ******************************
	RETURN 0;

}



int
HiLowFitsReaderWriter::writeArray(
	fitsfile *	fptr,
	const Array &	array,
	int &		status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	// ***************************************
	// Move to the appropriate fits extension.
	// ***************************************
	int			hdutype;
	const unsigned int	hdunum = blockNumberToHdunum( array );

	fits_movabs_hdu(
		fptr,
		hdunum,
		&hdutype,
		&status );
	fits_check(fptr);

	// **********************************************************
	// Revert the indices since FITS uses the Fortran convention,
	// i.e. the first index runs fastest.
	// **********************************************************
	//unsigned dimensions = array.dimensions();
	//long size[dimensions];
	//for (unsigned i = 0; i < dimensions; ++i)
	//	size[i] = array.size()[dimensions-1-i];

	// *************
	// Create Image.
	// *************
	//fits_create_img(
	//		fptr,
	//		toFITSBitpix( array.dataType() ),
	//		array.dimensions(),
	//		size,
	//		&status );

	// ********************
	// Set the array units.
	// ********************
	writeArrayUnits( fptr, array, status );


	// **********************************
	// write the array Scaling factors.
	// **********************************
	if( array.scaled() )
	{
		double		value;

		value = array.scaleZero();

		fits_write_key(
			fptr,
			TDOUBLE,
			"BZERO",
			&value,
			"",
			&status );

		value = array.scaleFactor();

		fits_write_key(
			fptr,
			TDOUBLE,
			"BSCALE",
			&value,
			"",
			&status );

		// ***********************************
		// Disable the FITS automatic scaling.
		// ***********************************
		fits_set_bscale(
				fptr,
				1.0,
				0.0,
				&status );

		fits_check(fptr);

	} // if

	// ***********************
	// Write array null value.
	// ***********************
	if( array.nullDefined() )
	{
		writeArrayNullValue( fptr, &array, status );
	}
	else
	{
		// =======================================
		// Delete the BLANK keyword (if it exists)
		// =======================================
		deleteUnwantedKeyword( fptr, "BLANK", status );
	} // if

	// ******************************
	// Everything is Ok, so return 0.
	// ******************************
	RETURN 0;

}


static int
readVariableLengthColumnData(
	fitsfile *	fptr,
	Column *	column,
	int &		columnNumber,
	int &		status )
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	for( unsigned int i = 0; i < column -> rows(); ++i )
	{
		// ***********************************************
		// Get the fits variable length column descriptor,
		// in order to determine how many elements are
		// contained in the current row.
		// Then check status.
		// ***********************************************
		long 	repeat;
		long 	offset;

		fits_read_descript(
				fptr,
				columnNumber + 1,
				i + 1,
				&repeat,
				&offset,
				&status );

		fits_check(fptr);

		// **************************************************
		// Get a column CellData descriptor, set its size.
		// As usual the CellData descriptor object must be
		// deleted when it is no longer needed.
		// **************************************************
		CellData * c = column -> cellData( i );
		c -> size( repeat );
		delete c;
	} // for

	// ******************************
	// Everything is Ok, so return 0.
	// ******************************
	RETURN 0;
}

/*
int
HiLowFitsReaderWriter::readColumn(
	fitsfile *	fptr,
	int &		columnNumber,
	Table &		table,
	int &		status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	char			name[ 32 ];
	int			typeCode;
	char			comment[ 1024 ];
	char			columnComment[ 1024 ];
	char			units[ 32 ];
	string			tableName;
	const unsigned int	maxdim = 4;
	int			naxis;
	long			naxes[ maxdim ];
	string			type = "TTYPE";
	Column::DataType 	dataType;
	Column *		column;


	// *************************************
	// Get the column name & column comment.
	// *************************************
	fits_read_key(
			fptr,
			TSTRING,
			(char *)( type + toString( columnNumber + 1 )).c_str(),
			name,
			columnComment,
			&status );

	// *******************************
	// Read the optional column units.
	// *******************************
	readColumnUnits( fptr, columnNumber, units, comment, status );


	// ***************
	// Get dimensions.
	// ***************
	fits_read_tdim(
			fptr,
			columnNumber + 1,
			maxdim,
			&naxis,
			naxes,
			&status );

	fits_check(fptr);

	// *************************
	// Get the column data type.
	// *************************
	getColumnDataType(
		fptr,
		columnNumber,
		typeCode,
		dataType,
		status );
	
	if( !isValidDalDataType( abs( typeCode )) )
	{
		errstr << "The column with column number " << columnNumber << " has a fits typecode " << typeCode << " which is not supported by the DAL" << Sas::error( "unsupportedColumnType" );
	} // if

	checkAndSetUnsignedColumn(
			fptr,
			columnNumber,
			dataType,
			status );


	// ********************************
	// Now add the column to the table.
	// ********************************
	
	if( typeCode >= 0 )
	{
		column = table.addColumn(
				name,
				dataType,
				columnComment,
				units,
				longVec(
					(unsigned long *)naxes,
					(unsigned long *)(naxes + naxis) ));
	}
	else
	{
		column = table.addColumn(
					name,
					dataType,
					columnComment,
					units,
					Dal::zero );

	} // if


	errstr << "added column '" << column->name() << "' to table '" << table.name() << "' type=" << typeCode << " dataType=" << int( dataType ) << " rows=" << column->rows() << " elements=" << column->elements() << " dimensions=" << column->dimensions() << Sas::message( Msg::Core, Msg::Verbose );

	if( !getenv( "SAS_NO_COLUMN_ATTRIBUTES" ) )
	{
		// ***********************
		// Read Column Attributes.
		// ***********************
		ColumnKeywordChecker * cac = new ColumnKeywordChecker( column );

		readAttributes(
			fptr,
			(Attributable &)*column,
			cac,
			status );

		delete cac;

	} // if

	// ***********************
	// TEMPORARILY DISABLED:
	// Get the Scaling Factors
	// ***********************
	//readColumnScalingFactors(
	//	fptr,
	//	columnNumber,
	//	*column,
	//	status );

	// **************************
	// Get the column null value.
	// **************************
	readColumnNullValue(
		fptr,
		*column,
		status );

	// ********************
	// Get the column data.
	// ********************
	if( typeCode >= 0 )
	{
		//readFixedLengthColumnData( fptr, column, columnNumber, status );
	}
	else
	{
		readVariableLengthColumnData( fptr, column, columnNumber, status );
	} // if


	// ******************************
	// Everything is Ok, so return 0.
	// ******************************
	RETURN 0;

}
*/
/*
int
HiLowFitsReaderWriter::readTable(
	fitsfile * 	fptr,
	int &		blockNumber,
	DataSet &	dataSet,
	int &		status ) const
{
	PROC

	long			rows;
	char 			card[ 1024 ];
	char 			comment[ 1024 ];
	int			columns;
	string			tableName;
	string			tableComment;

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if


	// ====================
	// Get the column size.
	// ====================
	fits_read_key_lng(
			fptr,
			"NAXIS2",
			&rows,
			comment,
			&status );

	fits_check(fptr);


	// ===================================================
	// Get the table name. This is achieved by getting the
	// FITS file keyword EXTNAME.
	// ===================================================
	getBlockNameAndComment( fptr, blockNumber, tableName, tableComment, status );


	// ******************************
	// Add the table to the data set.
	// ******************************
	Table * table = dataSet.addTable( tableName, rows, tableComment );

	errstr << "added table " << Sas::message( Msg::Core, Msg::Verbose );

	// ======================
	// read the table header.
	// ======================
	TableKeywordChecker * tac = new TableKeywordChecker( table );
	readHeader( fptr, (Attributable &)*table, tac, status );
	delete tac;

	// ============================================
	// Get the number of columns in the FITS table.
	// ============================================

	// -----------------------------------------------
	// Ensure that search is from top of table header.
	// -----------------------------------------------
	fits_read_record( fptr, 0, card, &status );

	fits_read_key(
			fptr,
			TINT,
			"TFIELDS",	// This is how to get no. cols.
			&columns,
			comment,
			&status );

	fits_check(fptr);

	// *****************
	// read each Column.
	// *****************
	for( int i = 0; i < columns; i++ )
	{
		readColumn(
				fptr,
				i,
				*table,
				status );

	} // for


	// ******************************
	// Everything is Ok, so return 0.
	// ******************************
	RETURN 0;

}
*/


int
HiLowFitsReaderWriter::readTable(
	fitsfile * 	fptr,
	DataSet &	dataSet,
	int &		status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	TableInfo	tableInfo;

	FitsReaderWriter::readTable( fptr, dataSet, tableInfo, status );

	// ********************
	// Get the column data.
	// ********************
	Table* 		table = dataSet.table( dataSet.blocks() - 1 );
	int		columns( table -> columns() );

	for( int i = 0; i < columns; ++i )
	{
		Column*		column( table -> column( i ) );

		if( column ->  cellType() == Column::Fixed )
		{
			//readFixedLengthColumnData( fptr, column, i, status );
		}
		else
		{
			readVariableLengthColumnData( fptr, column, i, status );
		} // if
	} // for

	// ******************************
	// Everything is Ok, so return 0.
	// ******************************
	RETURN 0;

}



int
HiLowFitsReaderWriter::readArray(
	fitsfile *	fptr,
	DataSet &	dataSet,
	int &		status ) const
{
	PROC

	string			arrayName;
	string			arrayComment;
	int			bitpix;
	Array::DataType		dataType;
	int			naxis;
	long			naxes[ 4 ];
	string			units;
	const unsigned int	blockNumber = dataSet.blocks();

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if


	// ===================================================
	// Get the array name. This is achieved by getting the
	// FITS file keyword EXTNAME.
	// ===================================================
	getBlockNameAndComment( fptr, blockNumber, arrayName, arrayComment, status );


	// ********************
	// Get array data type.
	// ********************
	getArrayDataType(
		fptr,
		bitpix,
		dataType,
		status );

	checkAndSetUnsignedArray(
		fptr,
		dataType,
		status );


	// *********************
	// read the array units.
	// *********************
	readArrayUnits( fptr, units, status );


	// ************************
	// Get the array dimensions
	// ************************
	readArrayDimensions( fptr, naxis, naxes, status );


	// *****************************
	// Add the array to the dataset.
	// *****************************
	Array * array = dataSet.addArray(
				arrayName,
				dataType,
				longVec(
					(unsigned long *)naxes,
					(unsigned long *)(naxes + naxis)),
				arrayComment,
				units );

	// **********************
	// Read array attributes.
	// **********************
	KeywordChecker * ac = new KeywordChecker;

	readHeader( fptr, (Attributable &)*array, ac, status );
	delete ac;

	readArrayScalingFactors(
		fptr,
		*array,
		status );

	// *************************
	// Get the array null value.
	// *************************
	readArrayNullValue(
		fptr,
		*array,
		status );

	// ******************************
	// Everything is Ok, so return 0.
	// ******************************
	RETURN 0;

}


int
HiLowFitsReaderWriter::deleteAttributes(
	fitsfile *		fptr,
	int &			status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	int keyNumber = 1;	// The first keyword in a table
				// has position 1.

	while( 1 )
	{
		char	keyName[ 24 ];
		char	comment[ 1024 ];
		char	value[ 1024 ];

		fits_read_keyn(
				fptr,
				keyNumber,
				keyName,
				value,
				comment,
				&status );

		// =======================================================
		// Either, there are no more keywords
		// (KEY_OUT_BOUNDS), in which case,
		// do not attempt to read any more
		// and exit from the loop, otherwise,
		// there was a real error.
		// =======================================================
		// -------------------------------------------------------
		// Strangely sometimes we get no error status but an empty
		// name after the last keyword.
		// -------------------------------------------------------
		// =======================================================
		if( status == KEY_OUT_BOUNDS || strlen( keyName ) == 0 )
		{
			status = 0;

			ALTRETURNV( 0 );
		} // if

		// ==============================================
		// If the keyword is reserved but not
		// "CONTINUE", "CREATOR", "DATE", "XDAL",
		// "XPROC" or "XMMSEP", then skip it.
		// ==============================================
		if(
			isFITSReservedKeyword( keyName )
			&&
			string( keyName ) != attributeSeparator
			&&
			string( keyName ) != "CONTINUE"
			&&
			string( keyName, 0, 5 ) != "XPROC"
			&&
			string( keyName ) != "DATE"
			&&
			string( keyName, 0, 4 ) != "XDAL"
			&&
			string( keyName ) != "CREATOR" )
		{
			++keyNumber;

			continue;

		} // if

		fits_delete_record( fptr, keyNumber, &status );
		//fits_delete_key( fptr, keyName, &status );
		fits_check(fptr);
	} // while

	// **********************************************
	// When we get here we have encountered an error.
	// **********************************************

	RETURN 1;
}


int
HiLowFitsReaderWriter::deleteProcessHistory(
	fitsfile *		fptr,
	int &			status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	int processNumber = 0;	// The first process has
				// has number 0.

	while( 1 )
	{
		char	xproc[ 9 ]; // "XPROCnnn" + null

		sprintf( xproc, "XPROC%d", processNumber );

		fits_delete_key(
				fptr,
				xproc,
				&status );

		// ------------------------------------
		// Either, there are no more xproc
		// keywords
		// (KEY_NO_EXIST), in which case,
		// do not attempt to read any more
		// and exit from the loop, otherwise,
		// there was a real error.
		// ------------------------------------
		if( status == KEY_NO_EXIST )
		{
			status = 0;

			ALTRETURNV( 0 );

		} // if

		fits_check(fptr);

		processNumber++;

	} // while

	// **********************************************
	// When we get here we have encountered an error.
	// **********************************************

	RETURN 1;
}


/*
int
HiLowFitsReaderWriter::deletePrimaryHeaderAttributes(
	fitsfile *		fptr,
	int &			status ) const
{
	PROC

	int keyNumber = 1;	// The first keyword in a table
				// has position 1.

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	while( 1 )
	{
		char	keyName[ 24 ];
		char	comment[ 1024 ];
		char	value[ 1024 ];

		fits_read_keyn(
				fptr,
				keyNumber,
				keyName,
				value,
				comment,
				&status );

		// =======================================================
		// Either, there are no more keywords
		// (KEY_OUT_BOUNDS), in which case,
		// do not attempt to read any more
		// and exit from the loop, otherwise,
		// there was a real error.
		// =======================================================
		// -------------------------------------------------------
		// Strangely sometimes we get no error status but an empty
		// name after the last keyword.
		// -------------------------------------------------------
		// =======================================================
		if( status == KEY_OUT_BOUNDS || strlen( keyName ) == 0 )
		{
			status = 0;

			ALTRETURNV( 0 );
		} // if

		// =============================================
		// If the keyword is not reserved or
		// if the keyword is "CONTINUE", then delete it.
		// =============================================
		if(
			!isFITSReservedKeyword( keyName )
			||
			strcmp( keyName, "CONTINUE" ) == 0 )
		{
			fits_delete_record( fptr, keyNumber, &status );
			fits_check(fptr);
		}
		else
		{
			++keyNumber;
		}// if
	} // while

	// **********************************************
	// When we get here we have encountered an error.
	// **********************************************

	RETURN 1;
}
*/


int
HiLowFitsReaderWriter::readPrimaryArray(
	fitsfile *		fptr,
	DataSet &		dataSet,
	KeywordChecker *	keywordChecker,
	int &			status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	const string	arrayComment = "Primary Array";
	int		bitpix;
	char		comment[ 1024 ];
	int		naxis;
	long		naxes[ 4 ];
	string		units;

	// *****************************************
	// Read the dimensions of the primary array.
	// If the naxis( number of dimensions) is
	// zero, then there is no primary image,
	// in which case, just exit.
	// *****************************************
	readArrayDimensions( fptr, naxis, naxes, status );

	if( naxis == 0 )
	{
		ALTRETURNV( 0 );
	} // if

	// *******************
	// read the data type.
	// *******************
	fits_read_key(
			fptr,
			TINT,
			"BITPIX",
			&bitpix,
			comment,
			&status );


	// *********************
	// Read the array units.
	// *********************
	readArrayUnits( fptr, units, status );



	// *****************************
	// Add the array to the dataset.
	// *****************************
	Array * array = dataSet.addArray(
				nameOfPrimaryImage,
				fromFITSBitpix( bitpix ),
				longVec(
					(unsigned long *)naxes,
					(unsigned long *)(naxes + naxis)),
				arrayComment,
				units );


	// **********************
	// Read array attributes.
	// **********************
	readHeader( fptr, (Attributable &)*array, keywordChecker, status );

	//deleteCommentsAndHistory( fptr, keywordChecker, status );

	// *********************
	// Read scaling factors.
	// *********************
	readArrayScalingFactors(
		fptr,
		*array,
		status );


	// ******************************
	// Everything is Ok, so return 0.
	// ******************************
	RETURN 0;
}


HiLowFitsReaderWriter::HiLowFitsReaderWriter()
{
	PROC
	ENDPROC
}



void
HiLowFitsReaderWriter::write( DataSet* dataSet )
{

	PROC

	_dataSet = dataSet;

	// ***********************
	// Open the DAL Data File.
	// ***********************
	const unsigned int	numberOfBlocks = _dataSet -> blocks();


	// ******************************
	// Open the (existing) FITS file.
	// ******************************
	if( !_fptr )
	{
		const string fileName = _dataSet -> name();

		fitsOpenFile( &_fptr, fileName.c_str(), READWRITE, &status );
		fits_check( _fptr );
	} // if

	// ***********************
	// Move to primary header.
	// ***********************
	int			hdutype;

	fits_movabs_hdu(
		_fptr,
		1,
		&hdutype,
		&status );
	fits_check(_fptr);

	//deletePrimaryHeaderAttributes( _fptr, status );
	deleteAttributes( _fptr, status );
	deleteCommentsAndHistory( _fptr, status );
	//deleteProcessHistory( _fptr, status );
	//deleteCreatorAndDate( _fptr, status );

	// **************************
	// Write the process history.
	// **************************
	writeProcessHistory( _fptr, _dataSet, status );

	// ************************
	// Write the debug history.
	// ************************
	writeDebugHistory( _fptr, _dataSet, status );

	// ***********************
	// Write Creator and Date.
	// ***********************
	writeCreatorAndDate( _fptr, _dataSet, status );

	// *****************************
	// write the DataSet attributes.
	// *****************************
	//writeHeader( _fptr, *_dataSet, "", 0, status );
	writeHeader( _fptr, (const Attributable&)*_dataSet, "", 0, status );

	// *******************************
	// For each Block in the Data Set:
	// *******************************
	for( unsigned int i = 0; i < numberOfBlocks; i++ )
	{
		const Block *b = _dataSet -> block(i);

		switch( b->type() )
		{
			case Block::TableT :
			{
				const Table *table = _dataSet -> table( i );
				writeTable( _fptr, *table, status );
			}
			break;

			case Block::ArrayT :
			{
				const Array *	array = _dataSet -> array( i );

				writeArray( _fptr, *array, status );

				if( i == 0 && array -> name() == nameOfPrimaryImage )
				{
					// ===================================
					// write DataSet attributes' separator
					// to the FITS primary header.
					// ===================================
					writeSeparator( _fptr, array -> attributes(), status );
				}
				else
				{
					deleteAttributes( _fptr, status );
					deleteCommentsAndHistory( _fptr, status );

					// ===================================================
					// Set the array name. This is achieved by setting the
					// FITS file keyword EXTNAME.
					// ===================================================
					fits_update_key_str(
						_fptr,
						"EXTNAME",	// This is how to set the name.
						(char *)array->name().c_str(),
						array->label()=="" ? (char*)defaultArrayComment.c_str() : (char *)array->label().c_str(),
						&status );
					fits_check( _fptr );
				} // if

				// ===========================
				// Write the array attributes.
				// ===========================
				writeHeader( _fptr, (const Attributable&)*array, "", 0, status );
			}
				break;

			default:
				errstr<<"could not write data set '" << _dataSet -> name() << "' since an invalid block type " << int( b->type() ) << " was found for block " << b->name() << Sas::error("internalError");

		} // switch
	} // for


	writeLongStringWarning( _fptr, status );


	// ****************
	// Close FITS file.
	// ****************
	fitsCloseFile( _fptr, &status );

	fits_check( _fptr );

	_fptr = 0;

	ENDPROC

}


void
HiLowFitsReaderWriter::read( DataSet* dataSet )
{

	PROC

	int		hdutype;
	char		value[ 1024 ];

	_dataSet = dataSet;

	const string	fileName = _dataSet -> name();

	// ******************************
	// Check that the dataset exists.
	// ******************************
	if( !dataSetServer -> exists( fileName ) )
	{
		errstr << "The dataset with name '" << fileName << "' could not be read; either it does not exist or it is in an unrecognised format" << Sas::error( "datasetCouldNotBeRead" );
	} // if

	// **********************************************
	// Open the FITS Data File.
	// Then check that the file opened without error.
	// **********************************************
	if( !_fptr )
	{
		//if( _dataSet -> mode() == Dal::Read )
		//{
			fitsOpenFile(&_fptr, fileName.c_str(), READONLY, &status );
		//}
		//else
		//{
		//	fits_open_file(&_fptr, fileName.c_str(), READWRITE, &status );
		//} // if
		fits_check(_fptr);
	} // if


	// **********************
	// Move to primary array.
	// **********************
	int hdunum = 1;			// Primary header in position 1.

	fits_movabs_hdu( _fptr, hdunum, &hdutype, &status );
	fits_check(_fptr);

	// *********************
	// Read process history.
	// *********************
	readProcessHistory( _fptr, _dataSet, status );

	// ***********************
	// Read the debug history.
	// ***********************
	readDebugHistory( _fptr, _dataSet, status );

	// **********************
	// Read Creator and Date.
	// **********************
	readCreatorAndDate( _fptr, _dataSet, status );

	// *************************************
	// Read primary array (if there is one).
	// *************************************
	PrimaryKeywordChecker * pkc = new PrimaryKeywordChecker();
	readPrimaryArray( _fptr, *_dataSet, pkc, status );
	delete pkc;

	// *****************************************************
	// read the data set attributes from the primary header.
	// *****************************************************
	DataSetKeywordChecker * dskc = new DataSetKeywordChecker();
	readHeader( _fptr, *_dataSet, dskc, status );
	//deleteCommentsAndHistory( _fptr, dskc, status );
	delete dskc;

	// ********************************
	// For each block in the FITS file.
	// ********************************

	// =======================
	// Begin with first block.
	// =======================
	hdunum = 2;			// First table in first extended unit.

	fits_movabs_hdu( _fptr, hdunum, &hdutype, &status );

	if (status && status != END_OF_FILE) fits_check( _fptr );

	// ===============
	// For each block.
	// ===============
	while( !status )
	{

		// ======================================
		// Check the block type. This is achieved
		// by getting the XTENSION (which is a
		// a mandatory keyword) keyword value:
		// BINTABLE - a binary table
		//
		// ======================================
		fits_read_keyword(
				_fptr,
				"XTENSION",
				value,
				(char *)0,
				&status );

		string cleanValue = trim(unquote(value));

		if(
			( cleanValue == "BINTABLE" )
			||
			( cleanValue ==  "ASCII" ) )
		{
			readTable( _fptr, *_dataSet, status );

		}
		else if( cleanValue == "IMAGE" )
		{
			readArray( _fptr, *_dataSet, status );

		}
		else
		{
			errstr << "Unrecognized XTENSION '" << value << "'"
			       << Sas::error( "FITSFailure" );

		} // if

		// =======================
		// Look for another block.
		// =======================
		hdunum++;

		fits_movabs_hdu( _fptr, hdunum, &hdutype, &status );

		if (status && status != END_OF_FILE) fits_check(_fptr);
	} // while

	// ****************
	// Close FITS file.
	// ****************
	status = 0;

	fitsCloseFile( _fptr, &status );

	fits_check(_fptr);

	_fptr = 0;

	ENDPROC

}

