// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include <Dal.h>
#include <errstr.h>

int main()
{
  DataSet* set = dataSetServer->open("test.dat",DataSet::Read);

  clog << *set << endl;

  Attribute* attr;

  attr = set->attribute("sbool");
  cout << *attr << endl;
  if (attr->asBool() || attr->label() != "set bool comment")
     Sas::error("testFailed","problem in set bool attribute");

  attr = set->attribute("sint");
  cout << *attr << endl;
  cout << attr->asInt() << ' ' << attr->units() << ' ' << attr->label() << endl;
  if (attr->asInt() != 1 || attr->units() != "int unit" || attr->label() != "set int comment")
    Sas::error("testFailed","problem in set int attribute");

  attr = set->attribute("sreal");
  cout << *attr << endl;
  if (attr->asReal() != 2.5 || attr->units() != "real unit" || attr->label() != "set real comment")
    Sas::error("testFailed","problem in set real attribute");

  attr = set->attribute("sstring");
cout << *attr << endl;  if (attr->asString() != "a set string" || attr->label() != "set string comment")
    Sas::error("testFailed","problem in set string attribute");


  Array* arr = set->array("array");

  attr = arr->attribute("abool");
  cout << *attr << endl;
  if (!attr->asBool() || attr->label() != "arr bool comment")
    Sas::error("testFailed","problem in array bool attribute");

  attr = arr->attribute("aint");
  cout << *attr << endl;
  if (attr->asInt() != 4 || attr->units() != "int unit" || attr->label() != "arr int comment")
    Sas::error("testFailed","problem in array int attribute");

  attr = arr->attribute("areal");
  cout << *attr << endl;
  if (attr->asReal() != 3.5 || attr->units() != "real unit" || attr->label() != "arr real comment")
    Sas::error("testFailed","problem in array real attribute");

  attr = arr->attribute("astring");
  cout << *attr << endl;
  if (attr->asString() != "an array string" || attr->label() != "arr string comment")
    Sas::error("testFailed","problem in array string attribute");


  Table* tab = set->table("table");

  attr = tab->attribute("abool");
  cout << *attr << endl;
  if (attr->asBool() || attr->label() != "tab bool comment")
    Sas::error("testFailed","problem in table bool attribute");

  attr = tab->attribute("aint");
  cout << *attr << endl;
  if (attr->asInt() != 7 || attr->units() != "int unit" || attr->label() != "tab int comment")
    Sas::error("testFailed","problem in table int attribute");

  attr = tab->attribute("areal");
  cout << *attr << endl;
  if (attr->asReal() != 5.5 || attr->units() != "real unit" || attr->label() != "tab real comment")
    Sas::error("testFailed","problem in table real attribute");

  attr = tab->attribute("astring");
  cout << *attr << endl;
  if (attr->asString() != "a table string" || attr->label() != "tab string comment")
    Sas::error("testFailed","problem in table string attribute");

  // ************************
  // Check column attributes.
  // ************************
  Column * col = tab->column("col1");
  attr = col->attribute("tlmin");
  cout << *attr << endl;
  if( attr->asInt() != 1 || attr->units() != "int unit" || attr->label() != "minimum")
    Sas::error("testFailed","problem in column attribute");

  attr = col->attribute("tlmax");
  cout << *attr << endl;
  if( attr->asInt() != 2 || attr->units() != "int unit" || attr->label() != "maximum")
    Sas::error("testFailed","problem in column attribute");

  col = tab->column("col2");
  attr = col->attribute("tlmin");
  cout << *attr << endl;
  if( attr->asInt() != 11 || attr->units() != "int unit" || attr->label() != "minimum")
    Sas::error("testFailed","problem in column attribute");

  attr = col->attribute("tlmax");
  cout << *attr << endl;
  if( attr->asInt() != 22 || attr->units() != "int unit" || attr->label() != "maximum")
    Sas::error("testFailed","problem in column attribute");
  dataSetServer->close(set);

  return 0;
}

