// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include<cassert>

#include <Dal.h>
#include "StopWatch.h"
#include <iostream>


static void
fill( DataSet* set, unsigned long nRows )
{

  Table* tab = set->addTable("some table",nRows);
  Column *col;
  ColumnData *d;
  uint32* ui32;

  col = tab->addColumn("uint32",Column::Uint32,"uint32 data","dm");

  for (unsigned long i = 0; i < nRows; ++i)
  {
    d = col->data(i,1);
    ui32 = d->uint32Data();
    ui32[ 0 ] = i;
    delete d;
    if( !( i % 100 ))
    {
    	cout << "fill[" << i << "]" << endl;
    } // if
  } // for
}


static void
check( DataSet* set, unsigned long nRows )
{
  Table* tab = set -> table("some table");
  Column *col;
  ColumnData *d;
  uint32* ui32;

  col = tab -> column("uint32");
  for (unsigned long i = 0; i < nRows; ++i)
  {
    d = col->data(i,1);
    ui32 = d->uint32Data();
    assert( ui32[0] == i );
    delete d;
    if( !( i % 100 ))
    {
    	cout << "check[" << i << "]" << endl;
    } // if
  } // for
}


static void
test( unsigned long nRows )
{
  // *********************************
  // Get clock for timing information.
  // *********************************
  StopWatch		cpuWatch( StopWatch::CPU );
  StopWatch		calenderWatch( StopWatch::Calender );

  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  fill( set, nRows );
  check( set, nRows );
  dataSetServer -> close( set );

  set = dataSetServer->open("test.dat",DataSet::Read);
  check( set, nRows );
  dataSetServer -> close( set );
  cout << "CPU time: " << cpuWatch.elapsed() << " secs" << endl;
  cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;
}


int
main()
{
  test( 10000 );
  //test( 1000000 );
}

