// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include<cassert>

#include <Dal.h>
#include <iostream>

int main()
{
  DataSet* set = dataSetServer->open("test.dat",DataSet::Create);
  Attribute * att;
  *set->addAttribute("sbool","set bool comment") = false;

  vector<unsigned long> size;
  size.push_back(1);
  size.push_back(2);
  size.push_back(3);
  Array* arr = set->addArray("array", Array::Int32, size);
  *arr->addAttribute("abool","arr bool comment") = true;

  Table* tab = set->addTable("table",10);
  *tab->addAttribute("tbool","tab bool comment") = false;

  Column* col = tab-> addColumn("col", Column::Int32 );
  *col->addAttribute("tlmin","minimum","int unit") = 1;

  cout << "qualified data set name = " <<  set -> qualifiedName() << endl;
  assert( set -> qualifiedName() == "test.dat" );
  att = set -> attribute( "sbool" );
  cout << "qualified data set attribute name = " <<  att -> qualifiedName() << endl;
  assert( att -> qualifiedName() == "test.dat%sbool" );

  cout << "qualified array name = " <<  arr -> qualifiedName() << endl;
  assert( arr -> qualifiedName() == "test.dat:array" );
  att = arr -> attribute("abool" );
  cout << "qualified array attribute name " << att -> qualifiedName() << endl;
  assert( att -> qualifiedName() == "test.dat:array%abool" );

  cout << "qualified table name = " <<  tab -> qualifiedName() << endl;
  assert( tab -> qualifiedName() == "test.dat:table" );
  att = tab -> attribute("tbool" );
  cout << "qualified table attribute name " << att -> qualifiedName() << endl;
  assert( att -> qualifiedName() == "test.dat:table%tbool" );

  cout << "qualified column name = " <<  col -> qualifiedName() << endl;
  assert( col -> qualifiedName() == "test.dat:table:col" );
  att = col -> attribute("tlmin" );
  cout << "qualified column attribute name " << att -> qualifiedName() << endl;
  assert( att -> qualifiedName() == "test.dat:table:col%tlmin" );
  
  dataSetServer->close(set);

  return 0;
}
