// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include <Dal.h>
#include <errstr.h>
#include<cassert>

int main()
{
  DataSet *set = dataSetServer->open("test.dat",DataSet::Create);

  unsigned long size[] = { 5,7,3,5,4,6 };
  unsigned long nullSize[] = { 0, 0 };

  Array* arr;
  Data* data;

// Null size array - PRIMARY HEADER
  arr = set->addArray("PRIMARY", Array::Int8,
	vector<unsigned long>(nullSize,nullSize+1));
  *(arr -> addAttribute( "ATT1", "an attribute", "-none-" )) = 123;
  *(arr -> addAttribute( "ATT2", "an attribute", "-none-" )) = 123;
 
  //data = arr->data();
  //int8* i8 = data->int8Data();
  //delete data;

// Null size array
  arr = set->addArray("int8null", Array::Int8,
	vector<unsigned long>(nullSize,nullSize+1));

  //data = arr->data();
  //int8* i8 = data->int8Data();
  //delete data;


// Boolean arrays are not supported.

  errstr << "int8" << Sas::message( Msg::Core, Msg::Verbose );
  arr = set->addArray("int8Array", Array::Int8,
	vector<unsigned long>(size,size+3));
  data = arr->data();
  int8* i8 = data->int8Data();
  for (unsigned i = 0; i < data->elements(); ++i)
    i8[i] = int(i%100);
  delete data;

  errstr << "uint16" << Sas::message( Msg::Core, Msg::Verbose );
  arr = set->addArray("uint16Array", Array::Uint16,
	vector<unsigned long>(size,size+1));
  data = arr->data();
  uint16* ui16 = data->uint16Data();
  for (unsigned i = 0; i < data->elements(); ++i)
    ui16[i] =(uint16)(i);
  delete data;

  errstr << "int16" << Sas::message( Msg::Core, Msg::Verbose );
  arr = set->addArray("int16Array", Array::Int16,
	vector<unsigned long>(size,size+1));
  data = arr->data();
  int16* i16 = data->int16Data();
  for (unsigned i = 0; i < data->elements(); ++i)
    i16[i] = int(i);
  delete data;

  errstr << "uint32" << Sas::message( Msg::Core, Msg::Verbose );
  arr = set->addArray("uint32Array", Array::Uint32,
	vector<unsigned long>(size,size+2));
  data = arr->data();
  uint32* ui32 = data->uint32Data();
  for (unsigned i = 0; i < data->elements(); ++i)
    ui32[i] = (uint32)(i);
  delete data;

  errstr << "int32" << Sas::message( Msg::Core, Msg::Verbose );
  arr = set->addArray("int32Array", Array::Int32,
	vector<unsigned long>(size,size+2));
  data = arr->data();
  int32* i32 = data->int32Data();
  for (unsigned i = 0; i < data->elements(); ++i)
    i32[i] = int(i);
  delete data;

  errstr << "real16" << Sas::message( Msg::Core, Msg::Verbose );
  arr = set->addArray("real32Array", Array::Real32,
	vector<unsigned long>(size,size+3));
  data = arr->data();
  real32* r32 = data->real32Data();
  for (unsigned i = 0; i < data->elements(); ++i)
    r32[i] = 0.5*i;
  delete data;

  errstr << "real64" << Sas::message( Msg::Core, Msg::Verbose );
  arr = set->addArray("real64Array", Array::Real64,
	vector<unsigned long>(size,size+1));
  data = arr->data();
  real64* r64 = data->real64Data();
  for (unsigned i = 0; i < data->elements(); ++i)
    r64[i] = 0.25*i;
  delete data;

// String arrays are not supported.

// Check the position() call.

  errstr << "fill with unique numbers" << Sas::message( Msg::Core, Msg::Verbose );

  arr = set->array("real32Array");
  ArrayData *adata = arr->data();
  real32 *a = adata->real32Data();
  unsigned n = 0;
  vector<unsigned long> idx(3);
  for (idx[0] = 0; idx[0] < size[0]; ++idx[0])
    for (idx[1] = 0; idx[1] < size[1]; ++idx[1])
      for (idx[2] = 0; idx[2] < size[2]; ++idx[2]) {
	unsigned long pos = adata->position(idx);
        clog << n << ' ' << pos << endl;
        assert( pos == n );
        assert( a[pos] == 0.5*n );
        ++n;
      }
  delete adata;

  dataSetServer->close(set);

  return 0;
}

