// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
#include<cassert>

#include <cstdio>


#include <Dal.h>
#include <sstream>
#include "StopWatch.h"
#include <iostream>


static void
create( const string& setName, unsigned long nRows )
{

  DataSet* set = dataSetServer->open(setName, DataSet::Create);
  Table* tab = set->addTable("some table",nRows);

  Column* col = tab->addColumn("real64",Column::Real64,"real64 data","Dm");
  ColumnData* d = col->data();
  real64 *r64 = d->real64Data();
  for (unsigned long i = 0; i < nRows; ++i)
    r64[i] = 0.25*i;
  delete d;

  dataSetServer->close(set);

}


static void
read( const string& setName, unsigned long nRows )
{
  DataSet* set = dataSetServer->open(setName, DataSet::Read);
  Table* tab = set->table("some table");
  Column* col = tab -> column("real64");
  ColumnData *d  = col -> data();
  real64 *r64 = d->real64Data();
  for (unsigned long i = 0; i < nRows; ++i)
    assert( r64[i] == 0.25*i );
  delete d;
  dataSetServer->close(set);
}


static void
test( const string& setName, unsigned long nRows )
{
	create( setName, nRows );

	// *********************************
	// Get clock for timing information.
	// *********************************
	StopWatch		cpuWatch( StopWatch::CPU );
	StopWatch		calenderWatch( StopWatch::Calender );

	read( setName, nRows );
	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
	cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;

	read( setName, nRows );
	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
	cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;

	dataSetServer -> keep( setName );

	read( setName, nRows );
	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
	cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;

	read( setName, nRows );
	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
	cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;

	dataSetServer -> discard( setName );
	read( setName, nRows );
	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
	cout << "Calender time: " << calenderWatch.elapsed() << " secs" << endl;

	read( setName, nRows );
	cout << "CPU time: " << cpuWatch.elapsed() / 1000000.0 << " secs" << endl;
}


int main()
{
	test( "test.dat", 10000000 );
}

