! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program test

  use dal
  use errorhandling

  implicit none

  type(DataSetT) set
  type(ArrayT) arr
  integer(kind=int32), dimension(:,:,:), pointer :: a
  integer(kind=int32), dimension(:), pointer :: b
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)
  integer :: i,j,k,n

  ! create a set
  set = dataSet("test.dat",CREATE)
  arr = addArray(set, "some table", INTEGER32, dimensions=s )
  call setAttribute(arr,"RA","(none)","RA of plate centre")

  a => int32array3Data(arr)

  ! test whether the shape of the returned array is identical to the shape
  ! originally specified
  if (any(shape(a) /= s)) then
    write(errstr,*) "shape of returned array",shape(a),"/= array spec",s
    call error("internalError")
  endif

  if (any(dimensions(arr) /= s)) then
    write(errstr,*) "dimensions of array",dimensions(arr),"/= array spec",s
    call error("internalError")
  endif

  ! fill with unique numbers
  n = 0
  do k=0,1
    do j=0,3
      do i=0,2 
        a(i,j,k) = n
	n = n + 1
      end do
    end do
  end do

  do k=0,1
    do j=0,3
     write(*,*) (a(i,j,k), i=0,2)
    end do
  end do


  call release(arr)

  ! access it again as vector
  arr = array(set,"some table",READ);
  b => int32data(arr)
  write(*,*) ( b(i), i=0,23)

  ! check the shape
  if ( any(shape(b) /= (/24/)) ) then
    write(errstr,*) "vector size",shape(b),"unequal to array size",s
    call error("internalError")
  endif
  
  ! check the values
  do i=0,23
    if ( b(i) /= i ) then
      write(errstr,*) "vector value",b(n),"unequal to array value",n
      call error("internalError")
    end if
  end do

  call release(arr)

  call release(set)

end program test
