! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.




subroutine testarray1()

  use dal
  use errorhandling
  implicit none

  integer, dimension(1), parameter :: size1 = (/ 20 /)

  real(kind=single), dimension(:), pointer :: r32_1, r32_2
  integer :: i = 0

  type(DataSetT) set1, set2
  type(ArrayT) array1, array2
  type(BlockT) block1

  ! *********************************
  ! test copyblock with vector column
  ! *********************************
  set1 = dataSet("test1.dat",CREATE)
  array1 = addArray(set1,"array1",Real32,units="dm", &
             dimensions=size1,comment="real32 array")
  r32_1 => real32VectorData( array1 )
  r32_1 = 37.123

  write(*,*) ( r32_1(i), i=0,19 )
  set2 = dataSet("test2.dat",CREATE)
  block1 = block( set1, "array1", READ )
  call copyBlock( set2, block1 )
  array2 = array( set2, "array1", READ )

  r32_2 => real32VectorData( array2 )
  r32_1 => real32VectorData( array1 )
  if( any( r32_2 /= r32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  call release(set1)
  call release(set2)
 
end subroutine



subroutine testarray2()

  use dal
  use errorhandling
  implicit none

  integer, dimension(2), parameter :: size1 = (/ 20, 30 /)

  real(kind=single), dimension(:,:), pointer :: r32_1, r32_2


  type(DataSetT) set1, set2
  type(ArrayT) array1, array2
  type(BlockT) block1

  ! *********************************
  ! test copyblock with vector column
  ! *********************************
  set1 = dataSet("test1.dat",CREATE)
  array1 = addArray(set1,"array1",Real32,units="dm", &
             dimensions=size1,comment="real32 array")
  r32_1 => real32Array2Data( array1 )
  r32_1 = 37.123

  set2 = dataSet("test2.dat",CREATE)
  block1 = block( set1, "array1", READ )
  call copyBlock( set2, block1 )
  array2 = array( set2, "array1", READ )

  r32_2 => real32Array2Data( array2 )
  r32_1 => real32Array2Data( array1 )
  if( any( r32_2 /= r32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  call release(set1)
  call release(set2)
 
end subroutine


subroutine testarray3()

  use dal
  use errorhandling
  implicit none

  integer, dimension(3), parameter :: size1 = (/ 10, 20, 30 /)

  real(kind=single), dimension(:,:,:), pointer :: r32_1, r32_2


  type(DataSetT) set1, set2
  type(ArrayT) array1, array2
  type(BlockT) block1

  ! *********************************
  ! test copyblock with vector column
  ! *********************************
  set1 = dataSet("test1.dat",CREATE)
  array1 = addArray(set1,"array1",Real32,units="dm", &
             dimensions=size1,comment="real32 array")
  r32_1 => real32Array3Data( array1 )
  r32_1 = 37.123

  set2 = dataSet("test2.dat",CREATE)
  block1 = block( set1, "array1", READ )
  call copyBlock( set2, block1 )
  array2 = array( set2, "array1", READ )

  r32_2 => real32Array3Data( array2 )
  r32_1 => real32Array3Data( array1 )
  if( any( r32_2 /= r32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  call release(set1)
  call release(set2)
 
end subroutine


subroutine testarray4column()

  use dal
  use errorhandling
  implicit none

  integer, dimension(4), parameter :: size1 = (/ 2,3,4,5 /)
  integer, dimension(4), parameter :: size2 = (/ 5,4,3,2 /)

  real(kind=single), dimension(:,:,:,:,:), pointer :: r32_1, r32_2
  integer(kind=int32), dimension(:,:,:,:,:), pointer :: i32_1, i32_2


  type(DataSetT) set1, set2
  type(TableT) table1, table2
  type(BlockT) block1
  type(ColumnT) col1, col2, col3, col4
  integer, parameter :: nRows = 10
  integer :: i

  ! *********************************
  ! test copyblock with vector column
  ! *********************************
  set1 = dataSet("test1.dat",CREATE)
  table1 = addTable(set1,"table1",10)
  col1 = addColumn(table1,"col1",Real32,units="dm", &
           dimensions=size1,comment="real32 column")
  r32_1 => real32Array4Data( col1 )
  r32_1 = 37.123
  col2 = addColumn(table1,"col2",Integer32,units="dm", &
           dimensions=size2, comment="int32 column")
  i32_1 => int32Array4Data(col2)
  i32_1 = 38

  set2 = dataSet("test2.dat",CREATE)
  block1 = block( set1, "table1", READ )
  call copyBlock( set2, block1 )
  table2 = table( set2, "table1" )

  col3 = column( table2, "col1", READ )
  r32_2 => real32Array4Data( col3 )
  r32_1 => real32Array4Data( col1 )
  if( any( r32_2 /= r32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  col4 = column( table2, "col2", READ )
  i32_2 => int32Array4Data( col4 )
  i32_1 => int32Array4Data(col2)
  if( any( i32_2 /= i32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  call release(set1)
  call release(set2)
 
end subroutine


subroutine testarray3column()

  use dal
  use errorhandling
  implicit none

  integer, dimension(3), parameter :: size1 = (/ 3,4,5 /)
  integer, dimension(3), parameter :: size2 = (/ 5,4,3 /)

  real(kind=single), dimension(:,:,:,:), pointer :: r32_1, r32_2
  integer(kind=int32), dimension(:,:,:,:), pointer :: i32_1, i32_2


  type(DataSetT) set1, set2
  type(TableT) table1, table2
  type(BlockT) block1
  type(ColumnT) col1, col2, col3, col4
  integer, parameter :: nRows = 10
  integer :: i

  ! *********************************
  ! test copyblock with vector column
  ! *********************************
  set1 = dataSet("test1.dat",CREATE)
  table1 = addTable(set1,"table1",10)
  col1 = addColumn(table1,"col1",Real32,units="dm", &
           dimensions=size1,comment="real32 column")
  r32_1 => real32Array3Data( col1 )
  r32_1 = 37.123
  col2 = addColumn(table1,"col2",Integer32,units="dm", &
           dimensions=size2, comment="int32 column")
  i32_1 => int32Array3Data(col2)
  i32_1 = 38

  set2 = dataSet("test2.dat",CREATE)
  block1 = block( set1, "table1", READ )
  call copyBlock( set2, block1 )
  table2 = table( set2, "table1" )

  col3 = column( table2, "col1", READ )
  r32_2 => real32Array3Data( col3 )
  r32_1 => real32Array3Data( col1 )
  if( any( r32_2 /= r32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  col4 = column( table2, "col2", READ )
  i32_2 => int32Array3Data( col4 )
  i32_1 => int32Array3Data(col2)
  if( any( i32_2 /= i32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  call release(set1)
  call release(set2)
 
end subroutine

subroutine testarray2column()

  use dal
  use errorhandling
  implicit none

  integer, dimension(2), parameter :: size1 = (/ 3,5 /)
  integer, dimension(2), parameter :: size2 = (/ 5,3 /)

  real(kind=single), dimension(:,:,:), pointer :: r32_1, r32_2
  integer(kind=int32), dimension(:,:,:), pointer :: i32_1, i32_2


  type(DataSetT) set1, set2
  type(TableT) table1, table2
  type(BlockT) block1
  type(ColumnT) col1, col2, col3, col4
  integer, parameter :: nRows = 10
  integer :: i

  ! *********************************
  ! test copyblock with vector column
  ! *********************************
  set1 = dataSet("test1.dat",CREATE)
  table1 = addTable(set1,"table1",10)
  col1 = addColumn(table1,"col1",Real32,units="dm", &
           dimensions=size1,comment="real32 column")
  r32_1 => real32Array2Data( col1 )
  r32_1 = 37.123
  col2 = addColumn(table1,"col2",Integer32,units="dm", &
           dimensions=size2, comment="int32 column")
  i32_1 => int32Array2Data(col2)
  i32_1 = 38

  set2 = dataSet("test2.dat",CREATE)
  block1 = block( set1, "table1", READ )
  call copyBlock( set2, block1 )
  table2 = table( set2, "table1" )

  col3 = column( table2, "col1", READ )
  r32_2 => real32Array2Data( col3 )
  r32_1 => real32Array2Data( col1 )
  if( any( r32_2 /= r32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  col4 = column( table2, "col2", READ )
  i32_2 => int32Array2Data( col4 )
  i32_1 => int32Array2Data(col2)
  if( any( i32_2 /= i32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  call release(set1)
  call release(set2)
 
end subroutine

subroutine testvectorcolumn()

  use dal
  use errorhandling
  implicit none

  integer, dimension(1), parameter :: size1 = (/ 10 /)
  integer, dimension(1), parameter :: size2 = (/ 5 /)

  real(kind=single), dimension(:,:), pointer :: r32_1, r32_2
  integer(kind=int32), dimension(:,:), pointer :: i32_1, i32_2


  type(DataSetT) set1, set2
  type(TableT) table1, table2
  type(BlockT) block1
  type(ColumnT) col1, col2, col3, col4
  integer, parameter :: nRows = 10
  integer :: i

  ! *********************************
  ! test copyblock with vector column
  ! *********************************
  set1 = dataSet("test1.dat",CREATE)
  table1 = addTable(set1,"table1",10)
  col1 = addColumn(table1,"col1",Real32,units="dm", &
           dimensions=size1,comment="real32 column")
  r32_1 => real32VectorData( col1 )
  r32_1 = 37.123
  col2 = addColumn(table1,"col2",Integer32,units="dm", &
           dimensions=size2, comment="int32 column")
  i32_1 => int32VectorData(col2)
  i32_1 = 38

  set2 = dataSet("test2.dat",CREATE)
  block1 = block( set1, "table1", READ )
  call copyBlock( set2, block1 )
  table2 = table( set2, "table1" )

  col3 = column( table2, "col1", READ )
  r32_2 => real32VectorData( col3 )
  r32_1 => real32VectorData( col1 )
  if( any( r32_2 /= r32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  col4 = column( table2, "col2", READ )
  i32_2 => int32VectorData( col4 )
  i32_1 => int32VectorData(col2)
  if( any( i32_2 /= i32_1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  call release(set1)
  call release(set2)
 
end subroutine


subroutine testscalarcolumn()

  use dal
  use errorhandling
  implicit none

  type(DataSetT) set1, set2
  type(TableT) table1, table2
  type(BlockT) block1
  type(ColumnT) col1, col2, col3, col4
  integer, parameter :: nRows = 10
  integer(kind=int32), dimension(:), pointer :: i1, i2, i3, i4
  integer :: i

  ! *********************************
  ! test copyblock with scalar column
  ! *********************************
  set1 = dataSet("test1.dat",CREATE)
  table1 = addTable(set1,"table1",15)
  col1 = addColumn(table1,"col1",Integer32,units="dm",comment="int32 column")
  i1 => int32Data( col1 )
  i1 = 37
  col2 = addColumn(table1,"col2",Integer32,units="dm",comment="int32 column")
  i2 => int32Data(col2)
  i2 = 38

  set2 = dataSet("test2.dat",CREATE)
  block1 = block( set1, "table1", READ )
  ! ======================================================
  ! copyBlock() has the effect of making all data pointers
  ! in the source and destination stale.
  ! ======================================================
  call copyBlock( set2, block1 )
  table2 = table( set2, "table1" )

  col3 = column( table2, "col1", READ )
  i3 => int32Data( col3 )
  i1 => int32Data( col1 ) ! original i1 has become stale
  if( any( i3 /= i1 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  col4 = column( table2, "col2", READ )
  i4 => int32Data( col4 )
  i2 => int32Data(col2) ! original i2 has become stale
  if( any( i4 /= i2 )) then
    call error("dalInternalError", "copy block failed" );
  end if

  call release(set1)
  call release(set2)
 
end subroutine


program test

  use dal
  use errorhandling
  implicit none

  !call testscalarcolumn()
  !call testvectorcolumn()
  !call testarray2column()
  !call testarray3column()
  !call testarray4column()
  call testarray1()
  !call testarray2()
  !call testarray3()

end program

