! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program test

  use dal
  use errorhandling

  implicit none

  type(DataSetT) :: set
  type(TableT) :: tab
  type(ColumnT) :: col
  logical(kind=BOOL), dimension(:), pointer :: b
  integer n, i, j

  set = dataSet("test.dat",Create)
  tab = addTable(set,"someTable",0)

  col = addColumn(tab,"bool",Boolean)
  col = addColumn(tab,"int8",Integer8)
  col = addColumn(tab,"int16",Integer16)
  col = addColumn(tab,"int32",Integer32)
  col = addColumn(tab,"real32",Real32)
  col = addColumn(tab,"real64",Real64)
  col = addColumn(tab,"string",String)

  n = 0
  do i=0,9

    call insertRows(tab,i,i)

    n = n + i

    if ( numberOfRows(tab) /= n ) then
	call error("test","wrong number of rows inserted")
    end if

    write(*,*) from( tab )
    if( from( tab ) /= 0 ) then
      call error("testFailed", "invalid from" )
    end if

    write(*,*) count( tab )
    if( count( tab ) /= numberOfRows( tab ) ) then
      call error("testFailed", "invalid count" )
    end if

    col = column( tab, "bool" )

    b => boolData( col )
    do j = 0, numberOfRows(tab) - 1
      write(*,*) b( j )
    end do
    call release( col )
  end do

  do i=0,4

    call deleteRows(tab,i,i)

    n = n - i

    if ( numberOfRows(tab) /= n) then
      call error("test","wrong number of rows deleted")
    end if

    write(*,*) from( tab )
    if( from( tab ) /= 0 ) then
      call error("testFailed", "invalid from" )
    end if

    write(*,*) count( tab )
    if( count( tab ) /= numberOfRows( tab ) ) then
      call error("testFailed", "invalid count" )
    end if

    col = column( tab, "bool" )

    b => boolData( col )
    do j = 0, numberOfRows(tab) - 1
      write(*,*) b( j )
    end do
    call release( col )
  end do

  call release(set)

end program

