! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

program test

  use dal
  
  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col1
  type(ColumnT) col2
  logical(kind=bool), dimension(:), pointer :: b1
  logical(kind=bool), dimension(:), pointer :: b2
  logical(kind=bool), dimension(:), pointer :: b3
  logical(kind=bool), dimension(:), pointer :: b4
  logical(kind=bool), dimension(:,:), pointer :: bv1
  logical(kind=bool), dimension(:,:), pointer :: bv2
  integer(kind=INT8), dimension(:), pointer :: i1
  integer(kind=INT8), dimension(:), pointer :: i2
  integer(kind=INT8), dimension(:,:), pointer :: iv1
  integer(kind=INT8), dimension(:,:), pointer :: iv2
  integer i
  integer j
  integer nRows
  character(20) setName
  character(20) tableName
  character(20) columnName1
  character(20) columnName2
  setName = "test.dat"
  tableName = "table"
  columnName1 = "column1"
  columnName2 = "column2"
  nRows = 10

  ! **************
  ! Scalar Boolean
  ! **************
  set = dataSet(setName,CREATE)
  tab = addTable(set,tableName,nRows)

  col1 = addColumn(tab,columnName1,BOOLEAN)
  b1 => boolData(col1)
  do i=0,numberOfRows(tab)-1
    b1(i) = ( modulo (i,2) .eq. 0 )
  end do

  call release(set)

  set = dataSet(setName,READ)
  tab = table(set,tableName)
  col1 = column(tab,columnName1,READ)
  b1 => boolData(col1)
  do i=0,numberOfRows(tab)-1
    if( b1(i) .neqv. ( modulo (i,2) .eq. 0 )) then
      call error("testFailed","value scalar boolean")
    end if
  end do

  call release(set)

  ! **************
  ! Vector Boolean
  ! **************
  set = dataSet(setName,CREATE)
  tab = addTable(set,tableName,nRows)
  col1 = addColumn(tab,columnName1,BOOLEAN,dimensions=(/5/))
  col2 = addColumn(tab,columnName2,BOOLEAN,dimensions=(/5/))
  b1 => boolData(col1)
  bv1 => boolVectorData(col2)
  do i=0,numberOfRows(tab)-1
    do j=0,numberOfElements(col1)-1
      b1(i*numberOfElements(col1)+j) = ( modulo (i,2) .eq. 0 )
      bv1(j,i) = ( modulo (i,2) .eq. 0 )
    end do
  end do

  call release( set )

  set = dataSet(setName,READ)
  tab = table(set,tableName)
  col1 = column(tab,columnName1,READ)
  col2 = column(tab,columnName2,READ)
  b1 => boolData(col1)
  b2 => boolData(col2)
  write(*,*) ( b1(i), i=0,49 )
  write(*,*) ( b2(i), i=0,49 )
  bv1 => boolVectorData(col1)
  bv2 => boolVectorData(col2)
  do i=0,9
      write(*,*) (bv1(j,i), j=0,4 )
  end do
  do i=0,9
      write(*,*) (bv2(j,i), j=0,4 )
  end do

  do i=0,numberOfRows(tab)-1
    do j=0,numberOfElements(col1)-1
      b3 => boolData(col1,i)
      write(*,*) b3(j)
      write(*,*) bv1(j,i)
      b4 => boolData(col2,i)
      write(*,*) b4(j)
      write(*,*) bv1(j,i)
      if( b1(i*numberOfElements(col1)+j) .neqv. ( modulo (i,2) .eq. 0 )) then
        call error("testFailed","value vector boolean")
      end if
      if( bv1(j,i) .neqv. ( modulo (i,2) .eq. 0 )) then
        call error("testFailed","value vector boolean")
      end if
      if( b1(i*numberOfElements(col1)+j) .neqv. b2(i*numberOfElements(col1)+j)) then
        call error("testFailed","value vector boolean")
      end if
      if( bv1(j,i) .neqv. bv2(j,i) ) then
        call error("testFailed","value vector boolean")
      end if
    end do
  end do

  call release(set)

  ! ***********
  ! Scalar Int8
  ! ***********
  set = dataSet(setName,CREATE)
  tab = addTable(set,tableName,nRows)

  col1 = addColumn(tab,columnName1,INTEGER8)
  i1 => int8Data(col1)
  do i=0,numberOfRows(tab)-1
    i1(i) = i
  end do

  call release(set)

  set = dataSet(setName,READ)
  tab = table(set,tableName)
  col1 = column(tab,columnName1,READ)
  i1 => int8Data(col1)
  do i=0,numberOfRows(tab)-1
    if( i1(i) /= i ) then
      call error("testFailed","value scalar integer8")
    end if
  end do

  call release(set)

  ! ***********
  ! Vector Int8
  ! ***********
  set = dataSet(setName,CREATE)
  tab = addTable(set,tableName,nRows)
  col1 = addColumn(tab,columnName1,INTEGER8,dimensions=(/5/))
  col2 = addColumn(tab,columnName2,INTEGER8,dimensions=(/5/))
  i1 => int8Data(col1)
  iv1 => int8VectorData(col2)
  do i=0,numberOfRows(tab)-1
    do j=0,numberOfElements(col1)-1
      i1(i*numberOfElements(col1)+j) = i*numberOfElements(col1)+j
      iv1(j,i) = i*numberOfElements(col1)+j
    end do
  end do

  call release( set )

  set = dataSet(setName,READ)
  tab = table(set,tableName)
  col1 = column(tab,columnName1,READ)
  col2 = column(tab,columnName2,READ)
  i1 => int8Data(col1)
  i2 => int8Data(col2)
  write(*,*) ( i1(i), i=0,49 )
  write(*,*) ( i2(i), i=0,49 )
  iv1 => int8VectorData(col1)
  iv2 => int8VectorData(col2)
  do i=0,9
    write(*,*) ( iv1(j,i), j=0,4 )
  end do

  do i=0,9
    write(*,*) ( iv2(j,i), j=0,4 )
  end do


  do i=0,numberOfRows(tab)-1
    do j=0,numberOfElements(col1)-1
      if( i1(i*numberOfElements(col1)+j) /= i*numberOfElements(col1)+j) then
        call error("testFailed","value vector boolean")
      end if
      if( iv1(j,i) /= i*numberOfElements(col1)+j) then
        call error("testFailed","value vector boolean")
      end if
      if( i1(i*numberOfElements(col1)+j) /= i2(i*numberOfElements(col1)+j)) then
        call error("testFailed","value vector boolean")
      end if
      if( iv1(j,i) /= iv2(j,i) ) then
        call error("testFailed","value vector boolean")
      end if
    end do
  end do

  call release(set)

end program test

