! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

subroutine test1( set, tab, tabAtts, arr, col )

  use dal

  type(DataSetT), intent(in) :: set
  type(TableT), intent(in) :: tab
  integer, intent(in) :: tabAtts
  type(ArrayT), intent(in) :: arr
  type(ColumnT), intent(in) :: col

  type(AttributeT) att



  att = attribute( set,0 )
  write(*,*) name( parent( att ))
  if( name( parent( att )) /= name( set )) then
	call error('internalError',"problem in parent method" )
  end if

  if( numberOfAttributes( tab ) /= tabAtts ) then
    call error( 'internalError',"Table has wrong number of attributes" )
  end if

  att = attribute( tab,0 )
  write(*,*) name( parent( att ))
  if( name( parent( att )) /= name( tab )) then
	call error('internalError',"problem in parent method" )
  end if

  write(*,*) name( parent( tab ))
  if( name( parent( tab )) /= name( set )) then
	call error('internalError',"problem in parent method" )
  end if

  att = attribute( arr,0 )
  write(*,*) name( parent( att ))
  if( name( parent( att )) /= name( arr )) then
	call error('internalError',"problem in parent method" )
  end if

  write(*,*) name( parent( arr ))

  if( name( parent( arr )) /= name( set )) then
	call error('internalError',"problem in parent method" )
  end if

  att = attribute( col,0 )
  write(*,*) name( parent( att ))
  if( name( parent( att )) /= name( col )) then
	call error('internalError',"problem in parent method" )
  end if

  write(*,*) name( parent( col ))

  if( name( parent( col )) /= name( tab )) then
	call error('internalError',"problem in parent method" )
  end if

  write(*,*) name( parent( parent( col )))

  if( name( parent( parent( col ))) /= name( set )) then
	call error('internalError',"problem in parent method" )
  end if

end subroutine test1


program test

  use dal
  
  implicit none

  type(DataSetT) set
  type(TableT) tab
  type(ColumnT) col
  type(ArrayT) arr
  integer, dimension(3), parameter :: s = (/ 3,4,2 /)
  character(len=1024) :: longstring

  longstring = "12345678901234567890123456789012345678901234567890123456789012345678901234567890"

  set = dataSet("test.dat",CREATE)
  call setAttribute(set,"sint8",1_int8,"int8 unit","set int8 comment")
  tab = addTable(set,"some table",100)
  call setAttribute(tab,"int8",1_int8,"int8 unit","set int8 comment")
  call setAttribute(tab,"string",longstring,"comment")
  arr = addArray(set, "some array", INTEGER32, dimensions=s )
  call setAttribute(arr,"sint8",1_int8,"int8 unit","set int8 comment")
  col = addColumn(tab,"bool",BOOLEAN)
  call setAttribute(col,"TLMIN",1_int8,"int8 unit","set int8 comment")

  call test1( set,tab,2,arr,col )

  call release(set)

  set = dataSet("test.dat",READ)
  tab = table(set,0)
  arr = array(set,1,READ)
  col = column(tab,0,READ)
  call test1( set,tab,2,arr,col )
  call release(set)

  set = dataSet("test.dat",MODIFY)
  tab = table(set,0)
  arr = array(set,1,READ)
  col = column(tab,0,READ)
  call test1( set,tab,2,arr,col )
  call release(set)

  set = dataSet("test.dat",READ)
  tab = table(set,0)
  arr = array(set,1,READ)
  col = column(tab,0,READ)
  call test1( set,tab,2,arr,col )
  call release(parent(tab))
  !call release(set)

  set = dataSet("test.dat",READ)
  tab = table(set,0)
  arr = array(set,1,READ)
  col = column(tab,0,READ)
  call test1( set,tab,2,arr,col )
  call release(set)

end program test
