! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
! Author: P.Videler
!
! Description: testing f90 interface to setData
! 
! Comments:   only testing signed int32 to signed int16 conversion
! 
! Modifications: 
! 
! $Id: setdata.f90,v 1.2 2004/03/05 14:43:04 eojero Exp $

program testsetdata

  use dal
  implicit none 

  type(DataSetT):: set
  type(tableT)  :: tbl
  type(columnT) :: i32col, i16col, i8col

  integer(kind=int32), dimension(:), pointer :: i32
  integer(kind=int16), dimension(:), pointer :: i16 
  integer(kind=int8),  dimension(:), pointer :: i8

  integer, parameter  :: nrows=10, myNull=137
  integer :: i

  set = dataset('tmp.ds',create) 
  tbl = addtable(set,'mytbl',nrows)
  i32col = addColumn(tbl, 'INT32', integer32)
  i16col = addColumn(tbl, 'INT16', integer16)
  i8col  = addColumn(tbl, 'INT8', integer8)

  i32 => int32Data(i32col) 
  i32 = (/(i, i=0, nrows-1, 1)/)
  i32 = i32 - nrows/2
  write(*,*) 'i32=',i32

  ! syntax is  setData(toColum, fromColumn)

  write(*,*) 'no overflow - should pass'
  call setData (i16col, i32col) 
  i16 => int16Data(i16col) 
  if (any(i16 /= i32)) call error('setdata','int32 -> int16 conversion failed')

  write(*,*) 'testing overflow - should write null values'
  i32 = i32 + 2**15
  write(*,*) 'i32=',i32
  call setNullValue(i16col, myNull)
  call setData(i16col, i32col) 
  i16 => int16Data(i16col) 
  write(*,*) 'i16=',i16

!!$  write(*,*) 'testing input has nuls - should fail'
!!$  i32(1)=myNull
!!$  write(*,*) 'i32=',i32
!!$  call setNullValue(i16col, myNull)
!!$  call setData(i16col, i32col) 
!!$  i16 => int16Data(i16col) 
!!$  write(*,*) 'i16=',i16


  write(*,*) 'testing overflow both nulldefined - should write null values'
  i32 => int32Data(i32col) 
  i32 = (/(i, i=0, nrows-1, 1)/)
  i32 = i32 - nrows/2
  i32 = i32 + 2**15
  call setNullValue(i32col, myNull)
  call setNullValue(i16col, myNull+1)
  call setData(i16col, i32col) 
  i16 => int16Data(i16col) 
  write(*,*) 'i16=',i16


  !write(*,*) 'testing impossible null values - should fail'
  !call setNullValue(i16col, 2**29)

  call release (set)

end program testsetdata

