// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
using namespace std;
//
// XMM Science Analysis System
//
// Giuseppe Vacanti (cosine science & computing) 
// September 2001
//
#include<Dal.h>
#include<errstr.h>
#include <ExceptionErrorHandler.h>  
#include<iostream>
#include<Tracer.h>
#include<cassert>

int main()
{

  //  tracer.trace(true);



  // SPR-2167
  // Column::insertRows() does not work for variable length vector
  // columns, in both high and low mem.

  dataSetServer->client("dal_test");

  const size_t nrows = 100;
  std::string set1 = "spr2167.ds";

  DataSet *s1 = dataSetServer->open(set1, Dal::Create);
  Table *t = s1->addTable("TABLE", nrows);  
  
  {
    Column * c = t->addColumn("real32",Dal::Real32,"","",Dal::zero);
    for(unsigned long k = 0; k < nrows; ++k){      
      CellData *d = c->cellData(k);
      d->size(k+1);
      real32 * v = d->real32Data();
      for(unsigned j = 0; j < d->size(); ++j)
	v[j] = k + j;    
      delete d;
    }
  }

  {
    longVec size(2,1);
    Column * c = t->addColumn("real32fixed",Dal::Real32,"","",size);
    for(unsigned long k = 0; k < nrows; ++k){      
      ColumnData *d = c->data();
      real32 * v = d->real32Data();
      for(unsigned j = 0; j < c->rows() * c->elements(); ++j)
	v[j] = k + j;    
      delete d;
    }
  }

  dataSetServer->close(s1);

  s1 = dataSetServer->open(set1, Dal::Modify);
  t = s1->table("TABLE");

  {
    CellData * cd = t->column("real32")->cellData(0);
    real32 * v = cd->real32Data();
    v[0] = 9999;
    delete cd;
  }
  t->insertRows(0, 1);

  dataSetServer->close(s1);

  s1 = dataSetServer->open(set1, Dal::Modify);
  t = s1->table("TABLE");
  CellData * cd = t->column("real32")->cellData(0);
  assert(cd->size() == 0);
  delete cd;
  cd = t->column("real32")->cellData(1);
  assert(cd->size() == 1);
  {
    real32 * v = cd->real32Data();
    assert(v[0] == 9999);
  }
  delete cd;
  cd = t->column("real32")->cellData(t->rows() - 1);
  assert(cd->size() == nrows);
  real32 * v = cd->real32Data();
  for(unsigned j = 0; j < cd->size(); ++j)
    assert(v[j] == nrows - 1 + j);
  delete cd;

  dataSetServer->close(s1);

}
