// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// SOC-SPR-2390: this fails in low mem
//
// Making NROWS and NELMS smaller makes the problem go away.
//
#include <Dal.h>

#include<iostream>

int main(int argc, char **) {

    std::string name("spr2390.ds");

    DataSet * set = dataSetServer->open(name, Dal::Create);

    const unsigned NROWS = 128;
    const unsigned NELMS = 20;
    const double c = 0.1;

    Table* table = set->addTable( "MATRIX", NROWS);
    

    {
	Column * vmc = table->addColumn("MATRIX",
					Column::Real32, 
					"No comment", 
					"", 
					Dal::zero);
	unsigned count = 0;
	for(unsigned k = 0; k < NROWS; ++k){
	    CellData * cd = vmc->cellData(k);
	    cd->size(NELMS);
	    real32 * v = cd->real32Data();
	    for(unsigned l = 0; l < NELMS; ++l)
		v[l] = c*count;
	    ++count;
	    delete cd;
	}
    }

// With these two it works.
    if(argc > 1){
	dataSetServer->close(set);
	set = dataSetServer->open(name, Dal::Modify);
    }

    // Adding the table first does not cause any problems.
    table = set->addTable( "EBOUNDS", 0);
    table->addColumn("CHANNEL", Column::Int16);

    dataSetServer->close(set);
}


